// https://xilinx-wiki.atlassian.net/wiki/spaces/A/pages/18842398/Linux+GPIO+Driver


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <malloc.h>
#include <string.h>
#include"gpio.h"


int open_gpio_channels(int gpio_base, gpio_pin_t *gpio_channel_list, unsigned char gpio_pins)
{
    char  gpio_nchan_file[128];
    int   gpio_nchan_fd;
    int   nchannel;
    char  nchannel_str[5];
    char  *cptr;
    char  channel_str[5];

    int   export_fd = 0;

    int   i;
    int   c;

    /* Check how many channels the GPIO chip has */
    sprintf(gpio_nchan_file, "%s/gpiochip%d/ngpio", GPIO_ROOT, gpio_base);
    gpio_nchan_fd = open(gpio_nchan_file, O_RDONLY);
    if (gpio_nchan_fd < 0) {
        fprintf(stderr, "Failed to open %s: %s\n", gpio_nchan_file, strerror(errno));
        return -1;
    }
    read(gpio_nchan_fd, nchannel_str, sizeof(nchannel_str));
    close(gpio_nchan_fd);


    nchannel = (int)strtoul(nchannel_str, &cptr, 0);
    if (cptr == nchannel_str) {
        fprintf(stderr, "Failed to change %s into GPIO channel number\n", nchannel_str);
        return -1;
    }

    /* Open files for each GPIO channel */
    export_fd=open(GPIO_EXPORT_FILE, O_WRONLY);
    if (export_fd < 0) {
        fprintf(stderr, "Cannot open GPIO to export %d\n", gpio_base);
        return -1;
    }

#if (0) // open all gpios
    gpio_max = gpio_base + nchannel;
    for(c = gpio_base; c < gpio_max; c++) {
        sprintf(channel_str, "%d", c);
        write(export_fd, channel_str, (strlen(channel_str) + 1));
    }
#else // open gpio channels in list
    for (i = 0; i < gpio_pins; i++) {
        c = gpio_base + gpio_channel_list[i].pin;
        sprintf(channel_str, "%d", c);
        write(export_fd, channel_str, (strlen(channel_str) + 1));
    }

    close(export_fd);

    char  gpio_dir_file[128];
    int   direction_fd = 0;

    for (i = 0; i < gpio_pins; i++) {
        c = gpio_base + gpio_channel_list[i].pin;
        sprintf(gpio_dir_file, "%s/gpio%d/direction", GPIO_ROOT, c);
        direction_fd=open(gpio_dir_file, O_RDWR);
        if (direction_fd < 0) {
            fprintf(stderr, "Cannot open the direction file for GPIO %d\n", c);
            return -1;
        }
        if (gpio_channel_list[i].dir == 1) {
            write(direction_fd, "out", 3+1);
        } else {
            write(direction_fd, "in", 2+1);
        }
        close(direction_fd);
    }

#endif

    return nchannel;
}


int close_gpio_channels(int gpio_base, gpio_pin_t *gpio_channel_list, unsigned char gpio_pins)
{
    char  gpio_nchan_file[128];
    int   gpio_nchan_fd;
    char  nchannel_str[5];
    char  *cptr;
    char  channel_str[5];

    int   unexport_fd = 0;

    int   i, c;

    /* Check how many channels the GPIO chip has */
    sprintf(gpio_nchan_file, "%s/gpiochip%d/ngpio", GPIO_ROOT, gpio_base);
    gpio_nchan_fd = open(gpio_nchan_file, O_RDONLY);
    if (gpio_nchan_fd < 0) {
        fprintf(stderr, "Failed to open %s: %s\n", gpio_nchan_file, strerror(errno));
        return -1;
    }
    read(gpio_nchan_fd, nchannel_str, sizeof(nchannel_str));
    close(gpio_nchan_fd);

    (int)strtoul(nchannel_str, &cptr, 0);
    if (cptr == nchannel_str) {
        fprintf(stderr, "Failed to change %s into GPIO channel number\n", nchannel_str);
//         exit(1);
        return -1;
    }

    /* Close opened files for each GPIO channel */
    unexport_fd = open(GPIO_UNEXPORT_FILE, O_WRONLY);
    if (unexport_fd < 0) {
        fprintf(stderr, "Cannot close GPIO by writing unexport %d\n", gpio_base);
        return -1;
    }

#if (0) // close all gpio channels
    gpio_max = gpio_base + nchannel;
    for(c = gpio_base; c < gpio_max; c++) {
        sprintf(channel_str, "%d", c);
        write(unexport_fd, channel_str, (strlen(channel_str) + 1));
    }
#else // close gpio channels in list
    for (i = 0; i < gpio_pins; i++) {
      c = gpio_base + gpio_channel_list[i].pin;
      sprintf(channel_str, "%d", c);
      write(unexport_fd, channel_str, (strlen(channel_str) + 1));
    }
#endif
    close(unexport_fd);

    return 0;
}


int set_gpio_value(int gpio_base, gpio_pin_t *gpio_channel_list, int pin, int value)
{
    char  gpio_val_file[128];
    int   val_fd = 0;
    char  val_str[2];
    int   c;

    c = gpio_base + gpio_channel_list[pin].pin;
    sprintf(gpio_val_file, "%s/gpio%d/value", GPIO_ROOT, c);
    val_fd = open(gpio_val_file, O_RDWR);
    if (val_fd < 0) {
        fprintf(stderr, "Cannot open the value file of GPIO %d\n", c);
        return -1;
    }

    sprintf(val_str,"%d", (value & 1));
    write(val_fd, val_str, sizeof(val_str));
    close(val_fd);

    return 0;
}


int get_gpio_value(int gpio_base, gpio_pin_t *gpio_channel_list, int pin)
{
    char  gpio_val_file[128];
    int   val_fd = 0;
    char  val_str[2];
    char  *cptr;
    int   value = 0;
    int   c;

    c = gpio_base + gpio_channel_list[pin].pin;
    sprintf(gpio_val_file, "%s/gpio%d/value", GPIO_ROOT, c);
    val_fd=open(gpio_val_file, O_RDWR);
    if (val_fd < 0) {
        fprintf(stderr, "Cannot open GPIO to export %d\n", c);
        return -1;
    }

    read(val_fd, val_str, sizeof(val_str));
    value = (int)strtoul(val_str, &cptr, 0);
    if (cptr == optarg) {
        fprintf(stderr, "Failed to change %s into integer", val_str);
    }
    close(val_fd);

    return value;
}
