/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7511_im.c
 * Date: 2022.03.03
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Avnet FMC HDMI OUT (ADV7511) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/


#include <stdio.h>
#include <string.h>
#include <unistd.h>
// #include <xil_io.h>
//
// #include <xiic.h>
// #include <xiic_l.h>

// #include "xparameters.h"
// #include <sleep.h>

#include "adv7511_im.h"

#include "../i2c/i2c_im.h"


// ***************************************************************************
const adv7511_im_i2c_t adv7511_im_dvi_init_conf[] =
{

    {'w', 0xD6, 0xC0},  // HPD is always high

    {'w', 0x15, 0x02}, // Input YCbCr 4:2:2 with embedded syncs
    {'w', 0x16, 0x38}, // 8, style 1, falling
//     {'w', 0x16, 0x78}, // 8, style 1, falling

    {'w', 0x48, 0x10}, // align right, left justified, DDR low bits, normal bus order
    {'w', 0x49, 0xA8}, // Bit trimming mode = 101010 (truncate)


    {'w', 0x55, 0x00}, // Set RGB in AVinfo Frame - R0x55[6:5] = Output Format = 00 (RGB)
    {'w', 0x56, 0x08}, //
    {'w', 0x57, 0xC8}, // 08
    {'w', 0x59, 0x10}, //


    {'w', 0x17, 0x04}, // DE gen disabled, aspec 4:3, linear interpolation

    {'w', 0xD0, 0x70}, // invert clock - Delay Adjust for the Input Video CLK Negative Edge for DDR Capture


//     {'w', 0xAF, 0x04}, // no HDCP
    {'w', 0xAF, 0x06}, // HDMI + HDCP

    {'w', 0x18, 0xC6},
//     {'w', 0x18, 0xE6},
    // Conversion matrix IN=OUT
//     {'w', 0x18, 0xA8},
//     {'w', 0x19, 0x00},
//     {'w', 0x1A, 0x00},
//     {'w', 0x1B, 0x00},
//     {'w', 0x1C, 0x00},
//     {'w', 0x1D, 0x00},
//     {'w', 0x1E, 0x00},
//     {'w', 0x1F, 0x00},
//     {'w', 0x20, 0x00},
//     {'w', 0x21, 0x00},
//     {'w', 0x22, 0x08},
//     {'w', 0x23, 0x00},
//     {'w', 0x24, 0x00},
//     {'w', 0x25, 0x00},
//     {'w', 0x26, 0x00},
//     {'w', 0x27, 0x00},
//     {'w', 0x28, 0x00},
//     {'w', 0x29, 0x00},
//     {'w', 0x2A, 0x00},
//     {'w', 0x2B, 0x00},
//     {'w', 0x2C, 0x08},
//     {'w', 0x2D, 0x00},
//     {'w', 0x2E, 0x00},
//     {'w', 0x2F, 0x00},
    // ------------------------


    {'w', 0x41, 0x10},

    {'w', 0x98, 0x03}, // FIXED 0x03
    {'w', 0x99, 0x02}, // ADI Recommended Write
    {'w', 0x9A, 0xE0}, // FIXED 0xE0
    {'w', 0x9C, 0x30}, // FIXED 0x30 PLL
    {'w', 0x9D, 0x61}, // xxxx_xx01 CLK Div
    {'w', 0xA2, 0xA4}, // FIXED
    {'w', 0xA3, 0xA4}, // FIXED
    {'w', 0xA5, 0x44}, // ADI Recommended Write
    {'w', 0xAB, 0x40}, // ADI Recommended Write

    {'w', 0xD1, 0xFF}, // ADI Recommended Write
    {'w', 0xDE, 0x9C}, // ADI Recommended Write
    {'w', 0xE0, 0xD0}, // FIXED
    {'w', 0xE4, 0x60}, // VCO_Swing_Reference_Voltage
    {'w', 0xF9, 0x00},

    {'w', 0xFF, 0xFF},

    {'w', 0xBA, 0x60}, // set no delay as default

    {0,0,0}
};


/*
 * embedded syncs
 */
adv7511_im_i2c_t adv7511_es_init_conf[] =
{
    // 1920x1080p60 is default
    {'w', 0x30, 0x16},
    {'w', 0x31, 0x02},
    {'w', 0x32, 0xC0},
    {'w', 0x33, 0x10},
    {'w', 0x34, 0x05},
    {'w', 0x17, 0x02},
    {0,0,0}
};


// ***************************************************************************



/*
 * configure ADV7511 chip
 */
int adv7511_im_init_sequence(int i2c_fd, unsigned char dev_addr, const adv7511_im_i2c_t conf[])
{
    unsigned int i = 0;
    int status;

    status = adv7511_im_i2c_sel(i2c_fd, I2C_ADV7511_IM_MUX_DATA);
    if (status < 0) return -1;

    while(conf[i].rw) {
        switch(conf[i].rw) {
            case 'r':
                status = adv7511_im_i2c_read(i2c_fd, dev_addr, conf[i].reg);
                if (status < 0) return -2;
                break;
            case 'w':
                status = adv7511_im_i2c_write(i2c_fd, dev_addr, conf[i].reg, conf[i].data);
                if (status < 0) return -3;
                break;
            default:
                break;
        }
        i++;
    }
    return 0;
}


/*
 * setp the configuration of embedded syncs according to required timing
 */
void adv7511_calc_embedded_syncs(adv7511_timing_t *tm, adv7511_im_i2c_t conf[])
{
    conf[0].data = (unsigned char)((tm->hfp >> 2) & 0xFF);
    conf[1].data = (unsigned char)((tm->hfp << 6) & 0xC0) |  (unsigned char)((tm->hsw >> 4) & 0x3F);
    conf[2].data = (unsigned char)((tm->hsw << 4) & 0xF0) |  (unsigned char)((tm->vfp >> 6) & 0x0F);
    conf[3].data = (unsigned char)((tm->vfp << 2) & 0xFC) |  (unsigned char)((tm->vsw >> 8) & 0x03);
    conf[4].data = (unsigned char)((tm->vsw >> 0) & 0xFF);
    conf[5].data = 0x00; // (unsigned char)( ... tm->hsp .. tm->vsp ... )
}


/*
 * set ADV7511 power up/down
 */
int adv7511_im_pd(int i2c_fd, unsigned char power_down )
{
    int status;
    unsigned char reg;

    status = adv7511_im_i2c_sel(i2c_fd, I2C_IO_EXP_IM_MUX_DATA);
    if (status < 0) return -1;

    status = adv7511_im_i2c_read(i2c_fd, I2C_EXP_ADV7511_IM_ADDR, IO_EXP_ADV7511_IM_PD_REG);
    if (status < 0) return -2;

    reg = (unsigned char)status;

    if ( power_down ) reg |=  IO_EXP_ADV7511_IM_PD_DATA;
    else              reg &= ~IO_EXP_ADV7511_IM_PD_DATA;

    status = adv7511_im_i2c_write(i2c_fd, I2C_EXP_ADV7511_IM_ADDR, IO_EXP_ADV7511_IM_PD_REG, reg);
    if (status < 0) return -3;

    return 0;
}


/*
 * adjust input clock delay (predefined values in its header)
 */
int adv7511_im_adjust_clkin_delay(int i2c_fd, unsigned char delay)
{
    int status;
    unsigned char val;

    status = adv7511_im_i2c_sel(i2c_fd, I2C_ADV7511_IM_MUX_DATA);
    if (status < 0) return -1;

    status = adv7511_im_i2c_read(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, ADV7511_IM_CLK_DELAY_REG);
    if ( status < 0 ) {
        return -2;
    }

    val = (unsigned char)status & 0x1F;
    val |= delay;

    status = adv7511_im_i2c_write(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, ADV7511_IM_CLK_DELAY_REG, val);
    if ( status < 0 ) {
        return -3;
    }

    return 0;
}


// ***************************************************************************
// ***************************************************************************

/*
 * initialize ADV7511 HDMI
 * I2C controller has to be already initialized
 */
int adv7511_im_init(int i2c_fd, adv7511_timing_t *timing, unsigned char clkin_delay, unsigned char hpd_test)
{
    int status = 0;
    int timeout;

    /*
     * power down and than power up the codec to reset it
     */
    status = adv7511_im_pd(i2c_fd, 1);
    if (status < 0) {
        printf("Cannot power down the codec ... ");
        return -1;
    }
    usleep(10000);
    status = adv7511_im_pd(i2c_fd, 0);
    if (status < 0) {
        printf("Cannot power up the codec ... ");
        return -2;
    }
    usleep(10000);


    /*
     * wait for HPD interrupt
     */

    // select ADV7511 I2C
    status = adv7511_im_i2c_sel(i2c_fd, I2C_ADV7511_IM_MUX_DATA);
    if (status < 0) {
        printf("Cannot switch I2C MUX to ADV7511 ... ");
        return -3;
    }

    // clear HPD interrupt
    status = adv7511_im_i2c_write(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, ADV7511_IM_EDID_INT_REG, ADV7511_IM_HPD_CLR);
    if (status < 0) {
        printf("Cannot clear ADV7511 HPD interrupt ... ");
        return -4;
    }

    // get new HPD
    if ( hpd_test ) {
        timeout = 100;
        status = 0;
        while ( ( status != ADV7511_IM_HPD ) && ( --timeout ) ) {
            status = adv7511_im_i2c_read(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, ADV7511_IM_EDID_INT_REG) & ADV7511_IM_HPD;
            if ( status != ADV7511_IM_HPD ) usleep(10);
        }
        if ( !timeout ) {
            printf("HDMI cable is unplugged ... ");
            return -5;
        }
    }

    // configure ADV7511
    status = adv7511_im_init_sequence(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, adv7511_im_dvi_init_conf);
    if (status < 0) {
        printf("Cannot configure the codec ... ");
        return -6;
    }

    /*
     * set embedded syncs generator
     */
    adv7511_calc_embedded_syncs(timing, adv7511_es_init_conf);
    status = adv7511_im_init_sequence(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, adv7511_es_init_conf);
    if (status < 0) {
        printf("Cannot configure embedded syncs ... ");
        return -7;
    }

    /*
     * adjust input clock delay (predefined values in its header)
     */
    status = adv7511_im_adjust_clkin_delay(i2c_fd, clkin_delay);
    if (status < 0) {
        printf("Cannot adjust inpt clock delay ... ");
        return -8;
    }

    // chip revision
    printf("rev. 0x%02X ... ", adv7511_im_i2c_read(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, 0x00));


    return 0;
}


