/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7511_im.h
 * Date: 2022.03.03
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Avnet FMC HDMI OUT (ADV7511) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/


#ifndef _ADV7511_IM_HEADER_FILE_
#define _ADV7511_IM_HEADER_FILE_

#ifdef __cplusplus
extern "C" {
#endif

#define I2C_ADV7511_IM_BASE_ADDR            0x39  //0x72

#define I2C_ADV7511_IM_MUX_ADDR             0x70  // 0xE0/0xE1 (PCA9546A)
#define I2C_ADV7511_IM_MUX_DATA             0x02  // ADV7511
#define I2C_IO_EXP_IM_MUX_DATA              0x08  // PCA9555 IO Expander

#define I2C_EXP_ADV7511_IM_ADDR             0x20  // 0x40/0x41 (PCA9534)
#define IO_EXP_ADV7511_IM_PD_REG            0x01
#define IO_EXP_ADV7511_IM_PD_DATA           0x10

#define ADV7511_IM_EDID_I2C_ADDR_REG        0x43
#define ADV7511_IM_EDID_INT_EN_REG          0x94
#define ADV7511_IM_EDID_INT_REG             0x96
#define ADV7511_IM_EDID_REREAD_REG          0xC9
#define ADV7511_IM_EDID_SEG_REG             0xC4

#define ADV7511_IM_EDID_INT_EN_MASK         0x04
#define ADV7511_IM_EDID_RDY_MASK            0x04
#define ADV7511_IM_EDID_REREAD_MASK         0x10
#define ADV7511_IM_EDID_HPD_INT_MASK        0x80
#define ADV7511_IM_EDID_HPD_MON_SENSE_MASK  0x40

#define ADV7511_IM_HPD                      (ADV7511_IM_EDID_HPD_INT_MASK | ADV7511_IM_EDID_HPD_MON_SENSE_MASK)
#define ADV7511_IM_HPD_CLR                  0x20

#define ADV7511_IM_HPD_TEST_ENABLED         1
#define ADV7511_IM_HPD_TEST_DISABLED        0

#define ADV7511_IM_CLK_DELAY_REG            0xBA
#define ADV7511_IM_CLK_DELAY_NEG_1_2_NS     0x00
#define ADV7511_IM_CLK_DELAY_NEG_0_8_NS     0x20
#define ADV7511_IM_CLK_DELAY_NEG_0_4_NS     0x40
#define ADV7511_IM_CLK_DELAY_0_0_NS         0x60
#define ADV7511_IM_CLK_DELAY_0_4_NS         0x80
#define ADV7511_IM_CLK_DELAY_0_8_NS         0xA0
#define ADV7511_IM_CLK_DELAY_1_2_NS         0xC0
#define ADV7511_IM_CLK_DELAY_1_6_NS         0xE0

typedef struct adv7511_im_i2c
{
    char            rw;
    unsigned char   reg;
    unsigned char   data;
} adv7511_im_i2c_t;


typedef struct {
    // horizontal timing
    unsigned int hav;  // active video
    unsigned int hfp;  // front porch
    unsigned int hsw;  // sync width
    unsigned int hbp;  // back porch
    unsigned int hsp;  // sync polarity

    // vertical timing
    unsigned int vav;  // active video
    unsigned int vfp;  // front porch
    unsigned int vsw;  // sync width
    unsigned int vbp;  // back porch
    unsigned int vsp;  // sync polarity
} adv7511_timing_t;


#include "../i2c/i2c_im.h"

#define adv7511_im_i2c_write  i2c_im_write
#define adv7511_im_i2c_read   i2c_im_read

#define adv7511_im_i2c_sel(fd, sel) i2c_im_sel(fd, sel)

int adv7511_im_init_sequence(int i2c_fd, unsigned char dev_addr, const adv7511_im_i2c_t conf[]);

void adv7511_calc_embedded_syncs(adv7511_timing_t *tm, adv7511_im_i2c_t conf[]);
int adv7511_im_pd(int i2c_fd, unsigned char power_down );
int adv7511_im_adjust_clkin_delay(int i2c_fd, unsigned char delay);

int adv7511_im_init(int i2c_fd, adv7511_timing_t *timing, unsigned char clkin_delay, unsigned char hpd_test);

#ifdef __cplusplus
}
#endif

#endif
