#ifndef _HDMI_IO_CTRL_HPP_
#define _HDMI_IO_CTRL_HPP_

//****************************************************************************
#include"vdma/vdma.h"
#define VDMA_DEVICE "vdma_1"
//#define VDMA_DEVICE "dma"
//#define VDMA_FB_NUM  3


#define MAX_RES_X       1920
#define MAX_RES_Y       1200          //1080
#define MAX_PIX_SIZE    4
#define MAX_FRAME_SIZE  (2048*2048*MAX_PIX_SIZE)     // max size of one frame for VDMA aligned to kB according to XAPP1205

#include <xrt.h>
#include <xrt/xrt_device.h>
#include <xrt/xrt_bo.h>


#include "xrt_util.hpp"


//****************************************************************************
#include"i2c/i2c_im.h"



//****************************************************************************
#include"clkwiz/clkwiz.h"
#define CLKWIZ_DEVICE "clk_wiz"



//****************************************************************************
#include"vtc/vtc.h"
#define VTC_DEVICE "v_tc"
extern vtc_timing_t vtc_resolutions[VTC_RES_CNT];


// ***************************************************************************
#include "hdmi/adv7611_im.h"
extern u8             adv7611_im_edid[256];


// ***************************************************************************
#include "hdmi/adv7511_im.h"

// ***************************************************************************
// video processing core - conversion
#include "csc/csc.h"
#define CSC_IN_DEVICE   "v_proc_ss_0"
#define CSC_OUT_DEVICE  "v_proc_ss_1"


extern XV_csc_Config XV_csc_ConfigTable[];


#include "stack.hpp"

class hdmi_io_ctrl {

	int verb;
	int framebuffers;
	unsigned *rq2term;
vdma_t  vdma;
xrt_obj_t *fb;
int i2c_fd;
im_reset_t i2c_rst = {
    334,
    78
}; // https://xilinx-wiki.atlassian.net/wiki/spaces/A/pages/18842398/Linux+GPIO+Driver
XAxiVdma_Config vdma_config = {
    0U,     //XPAR_VDMA_AXI_VDMA_0_DEVICE_ID,
    0U,     //XPAR_VDMA_AXI_VDMA_0_BASEADDR,
    3U,     //XPAR_VDMA_AXI_VDMA_0_NUM_FSTORES,
    1U,     //XPAR_VDMA_AXI_VDMA_0_INCLUDE_MM2S,
    0U,     //XPAR_VDMA_AXI_VDMA_0_INCLUDE_MM2S_DRE,
    64U,    //XPAR_VDMA_AXI_VDMA_0_M_AXI_MM2S_DATA_WIDTH,
    1U,     //XPAR_VDMA_AXI_VDMA_0_INCLUDE_S2MM,
    0U,     //XPAR_VDMA_AXI_VDMA_0_INCLUDE_S2MM_DRE,
    64U,    //XPAR_VDMA_AXI_VDMA_0_M_AXI_S2MM_DATA_WIDTH,
    0U,     //XPAR_VDMA_AXI_VDMA_0_INCLUDE_SG,
    1U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_VIDPRMTR_READS,
    1U,     //XPAR_VDMA_AXI_VDMA_0_USE_FSYNC,
    1U,     //XPAR_VDMA_AXI_VDMA_0_FLUSH_ON_FSYNC,
    4096U,  //XPAR_VDMA_AXI_VDMA_0_MM2S_LINEBUFFER_DEPTH,
    4096U,  //XPAR_VDMA_AXI_VDMA_0_S2MM_LINEBUFFER_DEPTH,
    3U,     //XPAR_VDMA_AXI_VDMA_0_MM2S_GENLOCK_MODE,
    2U,     //XPAR_VDMA_AXI_VDMA_0_S2MM_GENLOCK_MODE,
    1U,     //XPAR_VDMA_AXI_VDMA_0_INCLUDE_INTERNAL_GENLOCK,
    1U,     //XPAR_VDMA_AXI_VDMA_0_S2MM_SOF_ENABLE,
    16U,    //XPAR_VDMA_AXI_VDMA_0_M_AXIS_MM2S_TDATA_WIDTH,
    16U,    //XPAR_VDMA_AXI_VDMA_0_S_AXIS_S2MM_TDATA_WIDTH,
    0U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_1,
    0U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_5,
    1U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_6,
    1U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_7,
    0U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_9,
    0U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_13,
    1U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_14,
    1U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_INFO_15,
    0U,     //XPAR_VDMA_AXI_VDMA_0_ENABLE_DEBUG_ALL,
    32U,    //XPAR_VDMA_AXI_VDMA_0_ADDR_WIDTH,
    0U      //XPAR_VDMA_AXI_VDMA_0_ENABLE_VERT_FLIP
};
clkwiz_t clkwiz;
vtc_t vtc;
adv7611_im_timing_t   adv7611_tm;
adv7611_im_hot_plug_t adv7611_hp;
unsigned char         hdmii_reinit = 0;
adv7511_timing_t  adv7511_tm;

csc_t csc_in;
csc_t csc_out;

const char *i2c_im_device;



int readingFrame;
int showFrame;

int writingFrame;
int captureFrame;

bool isHdmiOutON;

xrtDeviceHandle xrt_dev;

unsigned fcnt;

public:

MyStack *fb_stack;

	hdmi_io_ctrl(int verb, const char *i2c_im_dev, int framebuffers, float prog_clkwiz_clk_in, unsigned *rq2term) {
		this->verb = verb;
		i2c_im_device = i2c_im_dev;
		this->framebuffers = framebuffers;

		adv7611_hp.is_cable_detected = 0;
		adv7611_hp.is_video_valid = 0;

		isHdmiOutON = false;

		fb_stack = new MyStack(framebuffers);

		clkwiz.clk_in = prog_clkwiz_clk_in;  // given by hardware design
		vdma_config.MaxFrameStoreNum = framebuffers;

		this->rq2term = rq2term;

		captureFrame = -1;
		writingFrame = -1;

		readingFrame = -1;
		showFrame    = -1;

		fcnt = 0;

		/*
		 * Initialize XRT device to obtain FB memory object allocatated by CMA
		 */
		if (verb > 0) printf("Open XRT device ... ");
		xrt_dev = xrtDeviceOpen(0);
		if (xrt_dev == NULL) {
			if (verb > 0) printf("FAILED. No valid device handle found. Make sure using right xclOpen index\n");
			   exit(0);
		}
		if (verb > 0) printf("OK\n");

		printf("Allocate FB storages ... \r\n");
		fb = new xrt_obj_t[framebuffers];
		for (int i = 0; i < framebuffers; i++) {
			fb[i].handle     = xrtBOAlloc(xrt_dev, MAX_FRAME_SIZE, 0, 0);
			fb[i].addr_virt  = (uintptr_t *)xrtBOMap(fb[i].handle);
			fb[i].addr       = (uintptr_t)xrtBOAddress(fb[i].handle);
			fb_stack->push(i);  // add frame to available frames
			if (verb > 0) printf("%08lX - %p OK\n", fb[i].addr, fb[i].addr_virt);
		}

		memset((void *)&adv7611_tm, 0, sizeof(adv7611_im_timing_t));
	}


	/*
	 * get VDMA errors
	 */
	int vdma_get_rx_err()
	{
	    uintptr_t addr = vdma.inst.BaseAddr;

	    // Get status of Error Flags
	    return *((volatile int *)(addr+XAXIVDMA_RX_OFFSET+XAXIVDMA_SR_OFFSET )) & 0x0000CFF0;
	}

	/*
	 * get VDMA errors
	 */
	int vdma_get_tx_err()
	{
	    uintptr_t addr = vdma.inst.BaseAddr;

	    // Get status of Error Flags
	    return *((volatile int *)(addr+XAXIVDMA_TX_OFFSET+XAXIVDMA_SR_OFFSET )) & 0x000046F0;
	}

	/*
	 * get VDMA errors
	 */
	void vdma_print_rx_err(int err_rx)
	{

	    printf( "VDMA - RX Error Flags detected:\n" );

		if ( err_rx & 0x00004000 ) printf( "\tS2MM_DMASR - ErrIrq\n"      );
		if ( err_rx & 0x00008000 ) printf( "\tS2MM_DMASR - EOLLateErr\n"  );
		if ( err_rx & 0x00000800 ) printf( "\tS2MM_DMASR - SOFLateErr\n"  );
		if ( err_rx & 0x00000400 ) printf( "\tS2MM_DMASR - SGDecErr\n"    );
		if ( err_rx & 0x00000200 ) printf( "\tS2MM_DMASR - SGSlvErr\n"    );
		if ( err_rx & 0x00000100 ) printf( "\tS2MM_DMASR - EOLEarlyErr\n" );
		if ( err_rx & 0x00000080 ) printf( "\tS2MM_DMASR - SOFEarlyErr\n" );
		if ( err_rx & 0x00000040 ) printf( "\tS2MM_DMASR - DMADecErr\n"   );
		if ( err_rx & 0x00000020 ) printf( "\tS2MM_DMASR - DMASlvErr\n"   );
		if ( err_rx & 0x00000010 ) printf( "\tS2MM_DMASR - DMAIntErr\n"   );
		if ( err_rx == 0) printf("none\n");
	}

	/*
	 * get VDMA errors
	 */
	void vdma_print_tx_err(int err_tx)
	{

		printf( "VDMA - TX Error Flags detected:\n" );

		if ( err_tx & 0x00004000 ) printf( "\tMM2S_DMASR - ErrIrq\n"      );
		if ( err_tx & 0x00000400 ) printf( "\tMM2S_DMASR - SGDecErr\n"    );
		if ( err_tx & 0x00000200 ) printf( "\tMM2S_DMASR - SGSlvErr\n"    );
		if ( err_tx & 0x00000080 ) printf( "\tMM2S_DMASR - SOFEarlyErr\n" );
		if ( err_tx & 0x00000040 ) printf( "\tMM2S_DMASR - DMADecErr\n"   );
		if ( err_tx & 0x00000020 ) printf( "\tMM2S_DMASR - DMASlvErr\n"   );
		if ( err_tx & 0x00000010 ) printf( "\tMM2S_DMASR - DMAIntErr\n"   );
		if ( err_tx == 0) printf("none\n");
	}


	int hdmi_out_start() {

		/*
		 * Initialize I2C for AVNET HDMII-HDMIO board
		 */
		if (verb > 0) printf("Initialize I2C controller for AVNET ... " );
		i2c_fd = i2c_im_init((char*)i2c_im_device, &i2c_rst);
		if ( i2c_fd < 0 ) {
	        if (verb > 0) printf("FAILED\n");
	        return -1;
	    }
		if (verb > 0) printf("%s ... OK\n", i2c_im_device);

		/*
		 * set programmable clock wizard - HDMI out pixel clock
		 */
		if (verb > 0) printf("Initialize Clock Wizard ... ");

		int status = clkwiz_init(&clkwiz, CLKWIZ_DEVICE);
		if (status < 0) {
			if (verb > 0 ) printf("FAILED %d\n", status);
			return -1;
	    }

		/*
		 * Setup maximal clock regardless resolution
		 */
		if ((clkwiz.clk_in == 50.0f) || (clkwiz.clk_in == 100.0f)) {
			if (verb > 0) printf("148.5 MHz ... ");
			status = clkwiz_set_clock(&clkwiz, CLKWIZ_148_5_MHZ);
			if (status < 0)  {
				if (verb > 0) printf("FAILED %d\n", status);
				return -1;
			}
			if (verb > 0) printf("OK\n");
		} else {
			printf("OK\n");
			printf("INFO: programmable clkwiz input clock %3.2f MHz not supported, skipping initialization (pixel clock remains at 148.5MHz)\r\n", clkwiz.clk_in);
		}

	    /*
	     * Initialize HDMI Input - ADV 7611 on AVNET board
	     */
	    if (verb > 0) printf("Initialize HDMI IN - ADV7611 ... ");
	    status = adv7611_im_init(i2c_fd, adv7611_im_edid);
	    if ( status < 0 ) {
	        if (verb > 0) printf("FAILED %d\n", status);
	        return -1;
	    }
	    if (verb > 0) printf("OK\n");


	    /*
	     * Initialize HDMI Output - ADV 7511 on AVNET board - 1920x1080p60
	     */
	    if (verb > 0) printf("Initialize HDMI OUT - ADV7511 ... " );
	    adv7511_tm.hav = vtc_resolutions[VTC_1920x1080p60].hav;
	    adv7511_tm.hfp = vtc_resolutions[VTC_1920x1080p60].hfp;
	    adv7511_tm.hsw = vtc_resolutions[VTC_1920x1080p60].hsw;
	    adv7511_tm.hbp = vtc_resolutions[VTC_1920x1080p60].hbp;
	    adv7511_tm.hsp = vtc_resolutions[VTC_1920x1080p60].hsp;

	    adv7511_tm.vav = vtc_resolutions[VTC_1920x1080p60].vav;
	    adv7511_tm.vfp = vtc_resolutions[VTC_1920x1080p60].vfp;
	    adv7511_tm.vsw = vtc_resolutions[VTC_1920x1080p60].vsw;
	    adv7511_tm.vbp = vtc_resolutions[VTC_1920x1080p60].vbp;
	    adv7511_tm.vsp = vtc_resolutions[VTC_1920x1080p60].vsp;

	    status = adv7511_im_init(i2c_fd, &adv7511_tm, ADV7511_IM_CLK_DELAY_NEG_0_8_NS, ADV7511_IM_HPD_TEST_DISABLED);
	    if ( status < 0 ) {
	        if (verb > 0) printf("FAILED\n");
	        return -1;
	    }
	    if (verb > 0) printf("OK\n");


	    /*
	     * initialize timing controller
	     */
	    if (verb > 0) printf("Initialize Timing Controller ... ");
	    status = vtc_init(&vtc, VTC_DEVICE);
	    if (status < 0) {
	        if (verb > 0) printf("FAILED %d\n", status);
	        return -1;
	    }

	    if (verb > 0) printf("common init ... ");
	    status = vtc_init_common(&vtc);
	    if (status < 0) {
	        if (verb > 0) printf("FAILED %d\n", status);
	        return -1;
	    }

	    vtc.timing_sel = VTC_1920x1080p60;
	    vtc_init_gen( &vtc );
	    if (verb > 0) printf(" %s ", vtc_resolutions[VTC_1920x1080p60].name);
	    if (verb > 0) printf("OK\n");

	    // HDMI IN
	    printf("Initialize CSC-IN ... ");
		status = csc_init(&csc_in, CSC_IN_DEVICE);
		if (status < 0) {
			printf("FAILED %d\n", status);
			return -1;
		}

		printf("common init ... ");
		status = csc_init_common(&csc_in, &XV_csc_ConfigTable[0]);
		if (status < 0) {
			printf("FAILED %d\n", status);
			return -1;
		}
		printf("OK\n");

		// HDMI out
		printf("Initialize CSC-OUT ... ");
		status = csc_init(&csc_out, CSC_OUT_DEVICE);
		if (status < 0) {
			printf("FAILED %d\n", status);
			return -1;
		}

		printf("common init ... ");
		status = csc_init_common(&csc_out, &XV_csc_ConfigTable[1]);
		if (status < 0) {
			printf("FAILED %d\n", status);
			return -1;
		}
		printf("OK\n");

		printf("Initialize CSC-OUT stream IN ... ");
		status = csc_set_stream_param(&csc_out,
									  CSC_STREAM_IN,
									  vtc_resolutions[VTC_1920x1080p60].hav,
									  vtc_resolutions[VTC_1920x1080p60].vav,
									  XVIDC_FR_60HZ,
									  XVIDC_CSF_RGB,
									  XVIDC_VF_PROGRESSIVE
							);
		if (status != 0) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}
		printf("Initialize CSC-OUT stream OUT ... ");
		status = csc_set_stream_param(&csc_out,
									  CSC_STREAM_OUT,
									  vtc_resolutions[VTC_1920x1080p60].hav,
									  vtc_resolutions[VTC_1920x1080p60].vav,
									  XVIDC_FR_60HZ,
									  XVIDC_CSF_YCRCB_422,
									  XVIDC_VF_PROGRESSIVE
							);
		if (status != 0) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}

		printf("Start CSC-OUT ... ");
		status = csc_setup_mode(&csc_out);
		if (status != 0) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}

	    /*
	     * Initialize VDMA
	     */
	    if (verb > 0)  printf("Initialize VDMA ... ");
	    status = vdma_init(&vdma, VDMA_DEVICE);
	    if (status < 0) {
	           if (verb > 0) printf("FAILED %d\n", status);
	           return -1;
	    }

	    vdma.tx_pix_bytes     = 3;
	    vdma.tx_timing.res.x  = vtc_resolutions[VTC_1920x1080p60].hav;
	    vdma.tx_timing.res.y  = vtc_resolutions[VTC_1920x1080p60].vav;
	    vdma.tx_offset.x      = 0;
	    vdma.tx_offset.y      = 0;
	    vdma.rx_pix_bytes     = 3;
		vdma.rx_timing.res.x  = vtc_resolutions[VTC_1920x1080p60].hav;
		vdma.rx_timing.res.y  = vtc_resolutions[VTC_1920x1080p60].vav;
		vdma.rx_offset.x      = 0;
		vdma.rx_offset.y      = 0;
	    vdma.fb_base          = fb[0].addr;
	    vdma.storage_res.x    = MAX_RES_X;  // always set to maximum
	    vdma.storage_res.y    = MAX_RES_Y;  // always set to maximum
	    vdma.fb_num           = framebuffers;

	    if (verb > 0) printf("common init ... ");
	    status = vdma_init_common(&vdma, &vdma_config);
	    if (status < 0) {
	       if (verb > 0) printf("FAILED %d\n", status);
	       return -1;
	    }

	    if (verb > 0) printf("TX Initialize ... ");
	    status = vdma_tx_init_fb( &vdma, fb, framebuffers);

	    if (status != 0) {
	        if (verb > 0) printf("FAILED\n");
	        return -1;
	    } else {
	        if (verb > 0) printf("OK\n");
	    }

#ifndef RD_PARKING_START
	    /* Start parking frames for output */
		if (verb > 0) printf("HDMI Out parking\r\n");

	    if (readingFrame == -1) readingFrame = fb_stack->pop();

	    vdma_clr_errors(&vdma);
		XAxiVdma_StartParking(&vdma.inst, readingFrame, XAXIVDMA_READ);

		/*
		 * wait until frame parked
		 */
		while ((u32)readingFrame != XAxiVdma_CurrFrameStore(&vdma.inst, XAXIVDMA_READ)) {
		    vdma_clr_errors(&vdma);
			XAxiVdma_StartParking(&vdma.inst, readingFrame, XAXIVDMA_READ);
		};

		if (showFrame == -1) showFrame = fb_stack->pop();
		XAxiVdma_StartParking(&vdma.inst, showFrame, XAXIVDMA_READ);vdma_clr_errors(&vdma);
#endif

   		if (verb > 0) printf("HDMI Out parking started\r\n");


   		isHdmiOutON = true;
   		return 0;
	}

	void hdmi_out_show(int frame) {
		/*
		 * wait until actual showFrame is shown (at least once)
		 */
		while ((u32)showFrame != XAxiVdma_CurrFrameStore(&vdma.inst, XAXIVDMA_READ)) {
			vdma_clr_errors( &vdma );
			XAxiVdma_StartParking(&vdma.inst, showFrame, XAXIVDMA_READ);
		};
		/*
		 * Release last reading frame
		 */
		fb_stack->push(readingFrame);

		/*
		 * Set new reading frame and new showframe
		 */
		readingFrame = showFrame;
		showFrame = frame;

		XAxiVdma_StartParking(&vdma.inst, showFrame, XAXIVDMA_READ);
	}

	int get_frame(void) {
		return fb_stack->pop();
	}

	uintptr_t *get_frame_addr(int f) {
		return fb[f].addr_virt;
	}

	void flush_frame(int f) {
		//xrtBOSync(fb.handle, XCL_BO_SYNC_BO_TO_DEVICE, MAX_FRAME_SIZE *  3, 0);
		xrtBOSync(fb[f].handle, XCL_BO_SYNC_BO_TO_DEVICE, MAX_FRAME_SIZE, 0);
	}

	void invalidate_frame(int f) {
		//xrtBOSync(fb.handle, XCL_BO_SYNC_BO_TO_DEVICE, MAX_FRAME_SIZE *  3, 0);
		xrtBOSync(fb[f].handle, XCL_BO_SYNC_BO_FROM_DEVICE, MAX_FRAME_SIZE, 0);
	}

	/*
	 * Checks the cable plugged event (HDMI input) and initialize HDMI input
	 */
	int check_hdmii_cable()
	{
	     return adv7611_im_cable_detect(i2c_fd);
	}

	/*
	 * Detects change of incomig video signals
	 */
	bool check_hdmii_resolution()
	{
	    if (adv7611_hp.is_video_valid && check_hdmii_cable()) {
	        (void)adv7611_im_get_timing(i2c_fd, &adv7611_tm);
	        if (
	            (vdma.rx_timing.res.x  != adv7611_tm.hav) ||  (vdma.rx_timing.res.y  != adv7611_tm.vav) ||
	            (vdma.rx_timing.hfp    != adv7611_tm.hfp) ||  (vdma.rx_timing.vfp    != adv7611_tm.vfp) ||
	            (vdma.rx_timing.hsw    != adv7611_tm.hsw) ||  (vdma.rx_timing.vsw    != adv7611_tm.vsw) ||
	            (vdma.rx_timing.hbp    != adv7611_tm.hbp) ||  (vdma.rx_timing.vbp    != adv7611_tm.vbp) ||
	            (vdma.rx_timing.hsp    != adv7611_tm.hsp) ||  (vdma.rx_timing.vsp    != adv7611_tm.vsp)
	        ) {
	            adv7611_hp.is_video_valid = 0;
	            return false;
	        }
	        return true;
	    }
	    return false;
	}

	bool check_hdmii_signal(vdma_resolution_t *d_res)
	{
    //    (void)adv7611_im_get_timing(i2c_fd, &adv7611_tm);
        if ((d_res->x  != adv7611_tm.hav) ||  (d_res->y != adv7611_tm.vav)) {
	        adv7611_hp.is_video_valid = 0;
	        printf("\n\r\tERROR : Input resolution does'n match to required resolution %dx%d\n\r", d_res->x, d_res->y);
	        return false;
	    }
		if (adv7611_tm.is_interlaced) {
			printf("\n\r\tERROR : Interlaced video signal is not supported!\n\r");
			return false;
		}

		if (adv7611_tm.is_encrypted) {
			printf("\n\r\tERROR : HDCP encrypted video signal is not supported!\n\r");
			return false;
		}
	    return true;
	}

	/*
	 * Set VDMA input according to currently detect input video signal parameters
	 */
	int set_hdmii_video_signal()
	{
	    int status;

		XV_CscStop(&(csc_in.inst));

	    printf("Initialize VDMA ... ");
		status = vdma_rx_stop( &(vdma.inst) );
		if (status < 0) {
			printf( "\r\nERROR : Cannot stop VDMA RX transfer ... FAILED\n" );
			return -1;
		}

		// Clear frame stores
//         clear_fb_YCrCb( vdma );

		// updete VDMA parameters
		vdma.rx_timing.res.x  = adv7611_tm.hav;
		vdma.rx_timing.hfp    = adv7611_tm.hfp;
		vdma.rx_timing.hsw    = adv7611_tm.hsw;
		vdma.rx_timing.hbp    = adv7611_tm.hbp;
		vdma.rx_timing.hsp    = adv7611_tm.hsp;

		vdma.rx_timing.res.y  = adv7611_tm.vav;
		vdma.rx_timing.vfp    = adv7611_tm.vfp;
		vdma.rx_timing.vsw    = adv7611_tm.vsw;
		vdma.rx_timing.vbp    = adv7611_tm.vbp;
		vdma.rx_timing.vsp    = adv7611_tm.vsp;

		vdma.rx_pix_bytes     = 3;
		vdma.rx_offset.x      = 0;
		vdma.rx_offset.y      = 0;

		// initialize VDMA input
		printf("RX Init ... ");
		status = vdma_rx_init_fb( &vdma, fb, framebuffers);
		if (status < 0) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}

#if (1)
		XV_CscStop(&(csc_in.inst));

		printf("Initialize CSC-IN stream IN ... ");
		status = csc_set_stream_param(&csc_in,
									  CSC_STREAM_IN,
									  adv7611_tm.hav,
									  adv7611_tm.vav,
									  XVIDC_FR_60HZ,
									  XVIDC_CSF_YCRCB_422,
									  XVIDC_VF_PROGRESSIVE
							);
		if (status != XST_SUCCESS) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}

		printf("Initialize CSC-IN stream OUT ... ");
		status = csc_set_stream_param(&csc_in,
									  CSC_STREAM_OUT,
									  adv7611_tm.hav,
									  adv7611_tm.vav,
									  XVIDC_FR_60HZ,
									  XVIDC_CSF_RGB,
									  XVIDC_VF_PROGRESSIVE
							);
		if (status != XST_SUCCESS) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}

		printf("Start CSC-IN ... ");
		status = csc_setup_mode(&csc_in);
		if (status != XST_SUCCESS) {
			printf( "FAILED\n" );
			return -1;
		} else {
			printf( "OK\n" );
		}
#endif
		// resolution changes have impact on errors flags - clear them
		vdma_clr_errors( &vdma );

		adv7611_hp.is_video_valid = 1;
		return 0;
	}




	int hdmi_in_start() {

		int hdmi_out_setup_attpempts = 3;
		while (!isHdmiOutON && (hdmi_out_setup_attpempts > 0)) {
			hdmi_out_start();
			hdmi_out_setup_attpempts--;
			if (*rq2term) return -1;
		}
		if (hdmi_out_setup_attpempts == 0) {
			printf("ERROR, cannot start HDMI output!!!\r\n");
			return -1;
		}

        if (!check_hdmii_cable()) {
        	printf("HDMI In unplugged\r\n");
        	return -1;
        }

        (void)adv7611_im_get_timing(i2c_fd, &adv7611_tm);
        adv7611_im_print_timing(&adv7611_tm);

        if (!check_hdmii_signal(&vdma.tx_timing.res)) {
        	printf("Detected incorrect HDMI input resolution\n");
        	return -1;
        }

        if (set_hdmii_video_signal()) {
        	printf("ERROR: Failed to start VDMA\r\n");
        	return -1;
        }

		/*
		 * Set parking frames for input
		 */

		if (verb > 0) printf("HDMI In parking start\r\n");

		if (captureFrame == -1) captureFrame = fb_stack->pop();
		if (writingFrame == -1) writingFrame = fb_stack->pop();


//		vdma_print_rx_err(vdma_get_rx_err());
//		vdma_print_tx_err(vdma_get_tx_err());
		vdma_clr_errors( &vdma );
		XAxiVdma_StartParking(&vdma.inst, captureFrame, XAXIVDMA_WRITE);
//		vdma_print_rx_err(vdma_get_rx_err());
//		vdma_print_tx_err(vdma_get_tx_err());
		/*
		 * wait until capture frame is being written, then move to writing frame
		 */
		while ((u32)captureFrame != XAxiVdma_CurrFrameStore(&vdma.inst, XAXIVDMA_WRITE)) {
			vdma_clr_errors( &vdma );
			XAxiVdma_StartParking(&vdma.inst, captureFrame, XAXIVDMA_WRITE);
			if (*rq2term) return -1;
		};
//		vdma_print_rx_err(vdma_get_rx_err());
//		vdma_print_tx_err(vdma_get_tx_err());
		XAxiVdma_StartParking(&vdma.inst, writingFrame, XAXIVDMA_WRITE);

		return 0;
	}

	int hdmi_in_capture(int &frame) {

#if (1)
		if (!check_hdmii_cable()) {
			adv7611_hp.is_cable_detected = 0;
			adv7611_hp.is_video_valid = 0;
			return -1;
		} else {
			//printf("Cable plugged\r\n");
		}
#endif
		int err = vdma_get_rx_err();
		if (err == 0) {
			/*
			 * Move capture process to another frame
			 */
			while ((u32)writingFrame != XAxiVdma_CurrFrameStore(&vdma.inst, XAXIVDMA_WRITE)) {
				vdma_clr_errors( &vdma );
				XAxiVdma_StartParking(&vdma.inst, writingFrame, XAXIVDMA_WRITE);
				if (*rq2term) return -1;
			};

			frame = captureFrame;

			captureFrame = writingFrame;
			writingFrame = fb_stack->pop();

			XAxiVdma_StartParking(&vdma.inst, writingFrame, XAXIVDMA_WRITE);
		//	printf("Got frame %d\r\n",fcnt++);
			return 0;
		} else {
			printf("VDMA error detected:");
			vdma_print_rx_err(err);
			printf("\r\n");
			adv7611_hp.is_cable_detected = 0;
			adv7611_hp.is_video_valid = 0;
			return -1;
		}
	}


	void print_info() {

		vdma_print_registers( &vdma );
		(void)vdma_check_err( &vdma );

		printf("VTC lock: %d\n", vtc_is_locked(&vtc.inst));
		printf("VTC status: 0x%08X\n", vtc_get_status(&vtc.inst));
		printf("VTC err: 0x%08X\n", vtc_is_locked(&vtc.inst));
		vtc_print_timing(&vtc);

		adv7611_im_i2c_sel(i2c_fd, I2C_ADV7511_IM_MUX_DATA);
        printf("0x%02X\n", adv7611_im_i2c_read(i2c_fd, I2C_ADV7511_IM_BASE_ADDR, 0x15));
	}


	~hdmi_io_ctrl() {

		printf("Exiting ... \r\n");
		vdma_rx_stop ( &(vdma.inst) );
		vdma_tx_stop ( &(vdma.inst) );
		XAxiVdma_ChannelReset( &(vdma.inst.ReadChannel) );
		XAxiVdma_ChannelReset( &(vdma.inst.WriteChannel) );

		printf("De-Initialize VDMA ... ");
		int status = vdma_release(&vdma);
		if (status < 0) {
			printf("FAILED %d\n", status);
		}
		printf("OK\n");

		printf("De-Init VTC ... ");
		status = vtc_release(&vtc);
		if (status < 0) {
			printf("FAILED %d\n", status);
		}
		printf("OK\n");

		printf("De-Init CLKWIZ ... ");
		status = clkwiz_release(&clkwiz);
		if (status < 0) {
			printf("FAILED %d\n", status);
		}
		printf("OK\n");

		printf("De-Init I2C ... ");
		i2c_im_deinit(&i2c_fd);
		printf("OK\n");

		// release buffers
		for (int i = 0; i < framebuffers; i++) {
			xrtBOFree(fb[i].handle);
		}
		delete fb;

		delete fb_stack;

		printf("Close XRT device ... ");
		status = xrtDeviceClose(xrt_dev);
		if (status < 0) {
			printf("FAILED %d\n", status);
		}
		printf("OK\n");


	}

};


#endif /* _HDMI_IO_CTRL_HPP_ */
