
#include <stdio.h>

//****************************************************************************
#include "dt/dt_fb.h"
#define FB_DT_DIR "/sys/firmware/devicetree/base/fbxserver"


//****************************************************************************
#include"vdma/vdma.h"
#define VDMA_DEVICE "vdma_0"
#define VDMA_FB_NUM 1

vdma_t  vdma;

XAxiVdma_Config vdma_config = {
    0U,     //XPAR_AXI_VDMA_0_DEVICE_ID,
    0U,     //XPAR_AXI_VDMA_0_BASEADDR,
    1U,     //XPAR_AXI_VDMA_0_NUM_FSTORES,
    1U,     //XPAR_AXI_VDMA_0_INCLUDE_MM2S,
    0U,     //XPAR_AXI_VDMA_0_INCLUDE_MM2S_DRE,
    128U,   //XPAR_AXI_VDMA_0_M_AXI_MM2S_DATA_WIDTH,
    0U,     //XPAR_AXI_VDMA_0_INCLUDE_S2MM,
    0U,     //XPAR_AXI_VDMA_0_INCLUDE_S2MM_DRE,
    64U,    //XPAR_AXI_VDMA_0_M_AXI_S2MM_DATA_WIDTH,
    0U,     //XPAR_AXI_VDMA_0_INCLUDE_SG,
    1U,     //XPAR_AXI_VDMA_0_ENABLE_VIDPRMTR_READS,
    1U,     //XPAR_AXI_VDMA_0_USE_FSYNC,
    1U,     //XPAR_AXI_VDMA_0_FLUSH_ON_FSYNC,
    2048U,  //XPAR_AXI_VDMA_0_MM2S_LINEBUFFER_DEPTH,
    512U,   //XPAR_AXI_VDMA_0_S2MM_LINEBUFFER_DEPTH,
    0U,     //XPAR_AXI_VDMA_0_MM2S_GENLOCK_MODE,
    0U,     //XPAR_AXI_VDMA_0_S2MM_GENLOCK_MODE,
    1U,     //XPAR_AXI_VDMA_0_INCLUDE_INTERNAL_GENLOCK,
    1U,     //XPAR_AXI_VDMA_0_S2MM_SOF_ENABLE,
    32U,    //XPAR_AXI_VDMA_0_M_AXIS_MM2S_TDATA_WIDTH,
    32U,    //XPAR_AXI_VDMA_0_S_AXIS_S2MM_TDATA_WIDTH,
    0U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_1,
    0U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_5,
    1U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_6,
    1U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_7,
    0U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_9,
    0U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_13,
    1U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_14,
    1U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_INFO_15,
    0U,     //XPAR_AXI_VDMA_0_ENABLE_DEBUG_ALL,
    32U,    //XPAR_AXI_VDMA_0_ADDR_WIDTH,
    0U      //XPAR_AXI_VDMA_0_ENABLE_VERT_FLIP
};


//****************************************************************************
#include"i2c/i2c_te.h"
#define I2C_TE_DEVICE "/dev/i2c-2"
int i2c_fd;

//****************************************************************************
// ***************************************************************************
// TE0701-05 HDMI out ADV7511 RGB 12b DDR
#include "hdmi/adv7511_te.h"


int main()
{
    int status;
    dt_fb_params_t fb_params;

    (void)get_dt_fb_params(FB_DT_DIR, &fb_params);

    printf("Initialize TE0701 HDMI OUT ...\n");

//    print_dt_fb_params(&fb_params);




    /*
     * Initialize I2C for AVNET HDMII-HDMIO board
     */
    printf("  Initialize I2C controller ... " );
    i2c_fd = i2c_te_init(I2C_TE_DEVICE);
    if ( i2c_fd < 0 ) {
        printf("FAILED\n");
        return -1;
    }
    printf("%s ... OK\n", I2C_TE_DEVICE);


    printf("  Initialize HDMI OUT - ADV7511 ... " );
    status = adv7511_te_init(i2c_fd, ADV7511_TE_CLK_DELAY_NEG_0_8_NS, ADV7511_TE_HPD_TEST_DISABLED);
    if ( status != 0 ) {
        printf("FAILED\n");
        return -1;
    } else {
        printf("OK\n");
    }


    /*
     * Initialize VDMA
     */
    printf("  Initialize VDMA ... ");
    status = vdma_init(&vdma, VDMA_DEVICE);
    if (status < 0) {
        printf("FAILED %d\n", status);
        return -1;
    }

    vdma.tx_pix_bytes     = 4;
    vdma.tx_timing.res.x  = fb_params.width;
    vdma.tx_timing.res.y  = fb_params.height;
    vdma.tx_offset.x      = 0;
    vdma.tx_offset.y      = 0;
    vdma.fb_base          = fb_params.base_low;
    vdma.storage_res.x    = fb_params.width;  // always set to maximum
    vdma.storage_res.y    = fb_params.height;  // always set to maximum
    vdma.fb_num           = VDMA_FB_NUM;


    printf("common init ... ");
    status = vdma_init_common(&vdma, &vdma_config);
    if (status < 0) {
        printf("FAILED %d\n", status);
        return -1;
    }

    printf("TX Initialize ... ");
    status = vdma_tx_init( &vdma );
    if (status != 0) {
        printf("FAILED\n");
        return -1;
    } else {
        printf("OK\n");
    }


    printf("  Release VDMA ... ");
    status = vdma_release(&vdma);
    if (status < 0) {
        printf("FAILED %d\n", status);
        return -1;
    }
    printf("OK\n");



    printf("  Release I2C ... ");
    i2c_te_deinit(&i2c_fd);
    printf("OK\n");

    printf("OK\n");


    return 0;
}
