# Update IP catalog
update_ip_catalog -rebuild

#activate extensible platform
set_property platform.extensible true [current_project]
save_bd_design

set_property PFM_NAME [string map {part0 zusys} [string map {trenz.biz trenz} [current_board_part]]] [get_files zusys.bd]
set_property platform.design_intent.embedded {true} [current_project]
set_property platform.design_intent.datacenter {false} [current_project]
set_property platform.design_intent.server_managed {false} [current_project]
set_property platform.design_intent.external_host {false} [current_project]
set_property platform.default_output_type {sd_card} [current_project]
set_property platform.uses_pr {false} [current_project]
save_bd_design

#add clocking wizard
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 clk_wiz_0
endgroup

#clocking wizard config
set_property -dict [list \
  CONFIG.CLKOUT2_USED {true} \
  CONFIG.CLKOUT3_USED {true} \
  CONFIG.CLKOUT4_USED {true} \
  CONFIG.CLKOUT2_REQUESTED_OUT_FREQ {200.000} \
  CONFIG.CLKOUT3_REQUESTED_OUT_FREQ {400.000} \
  CONFIG.CLKOUT4_REQUESTED_OUT_FREQ {240.000} \
  CONFIG.MMCM_CLKOUT1_DIVIDE {6} \
  CONFIG.MMCM_CLKOUT2_DIVIDE {3} \
  CONFIG.MMCM_CLKOUT3_DIVIDE {5} \
  CONFIG.NUM_OUT_CLKS {4} \
  CONFIG.CLKOUT2_JITTER {102.086} \
  CONFIG.CLKOUT2_PHASE_ERROR {87.180} \
  CONFIG.CLKOUT3_JITTER {90.074} \
  CONFIG.CLKOUT3_PHASE_ERROR {87.180} \
  CONFIG.CLKOUT4_JITTER {98.767} \
  CONFIG.CLKOUT4_PHASE_ERROR {87.180} \
  CONFIG.RESET_TYPE {ACTIVE_LOW} \
  CONFIG.RESET_PORT {resetn} \
] [get_bd_cells clk_wiz_0]

#connect clocking wizard inputs
connect_bd_net [get_bd_pins clk_wiz_0/resetn] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]
connect_bd_net [get_bd_pins clk_wiz_0/clk_in1] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]

#add reset cores
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 proc_sys_reset_1
create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 proc_sys_reset_2
create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 proc_sys_reset_3
create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 proc_sys_reset_4
endgroup

#connect reset cores
connect_bd_net [get_bd_pins clk_wiz_0/clk_out1] [get_bd_pins proc_sys_reset_1/slowest_sync_clk]
connect_bd_net [get_bd_pins clk_wiz_0/clk_out2] [get_bd_pins proc_sys_reset_2/slowest_sync_clk]
connect_bd_net [get_bd_pins clk_wiz_0/clk_out3] [get_bd_pins proc_sys_reset_3/slowest_sync_clk]
connect_bd_net [get_bd_pins clk_wiz_0/clk_out4] [get_bd_pins proc_sys_reset_4/slowest_sync_clk]
connect_bd_net [get_bd_pins clk_wiz_0/locked] [get_bd_pins proc_sys_reset_1/dcm_locked]
connect_bd_net [get_bd_pins clk_wiz_0/locked] [get_bd_pins proc_sys_reset_2/dcm_locked]
connect_bd_net [get_bd_pins proc_sys_reset_3/dcm_locked] [get_bd_pins clk_wiz_0/locked]
connect_bd_net [get_bd_pins proc_sys_reset_4/dcm_locked] [get_bd_pins clk_wiz_0/locked]
connect_bd_net [get_bd_pins proc_sys_reset_1/ext_reset_in] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]
connect_bd_net [get_bd_pins proc_sys_reset_2/ext_reset_in] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]
connect_bd_net [get_bd_pins proc_sys_reset_3/ext_reset_in] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]
connect_bd_net [get_bd_pins proc_sys_reset_4/ext_reset_in] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]

# add clocks to platform
set_property PFM.CLOCK {clk_out1 {id "1" is_default "false" proc_sys_reset "/proc_sys_reset_1" status "fixed" freq_hz "100000000"} clk_out2 {id "2" is_default "false" proc_sys_reset "/proc_sys_reset_2" status "fixed" freq_hz "200000000"} clk_out3 {id "3" is_default "false" proc_sys_reset "/proc_sys_reset_3" status "fixed" freq_hz "400000000"} clk_out4 {id "4" is_default "true" proc_sys_reset "/proc_sys_reset_4" status "fixed" freq_hz "240000000"}} [get_bd_cells /clk_wiz_0]

# prepare LPD interface for 240MHz for interrupt controller
disconnect_bd_net /zynq_ultra_ps_e_0_pl_clk1 [get_bd_pins zynq_ultra_ps_e_0/maxihpm0_lpd_aclk]
connect_bd_net [get_bd_pins clk_wiz_0/clk_out4] [get_bd_pins zynq_ultra_ps_e_0/maxihpm0_lpd_aclk]

#add interrupt core
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_intc:4.1 axi_intc_0
# endgroup

#config interrupt core
set_property -dict [list CONFIG.C_KIND_OF_INTR.VALUE_SRC USER] [get_bd_cells axi_intc_0]
set_property -dict [list CONFIG.C_KIND_OF_INTR {0x00000000} CONFIG.C_IRQ_CONNECTION {1}] [get_bd_cells axi_intc_0]

#connect interrupt core
connect_bd_net [get_bd_pins axi_intc_0/s_axi_aclk] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins axi_intc_0/s_axi_aresetn] [get_bd_pins proc_sys_reset_4/peripheral_aresetn]

startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect:2.1 axi_interconnect_0
endgroup
set_property -dict [list CONFIG.NUM_MI {1}] [get_bd_cells axi_interconnect_0]
connect_bd_net [get_bd_pins axi_interconnect_0/ACLK] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins axi_interconnect_0/ARESETN] [get_bd_pins proc_sys_reset_4/peripheral_aresetn]
connect_bd_net [get_bd_pins axi_interconnect_0/S00_ARESETN] [get_bd_pins proc_sys_reset_4/interconnect_aresetn]
connect_bd_net [get_bd_pins axi_interconnect_0/M00_ARESETN] [get_bd_pins proc_sys_reset_4/interconnect_aresetn]
connect_bd_net [get_bd_pins axi_interconnect_0/S00_ACLK] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins axi_interconnect_0/M00_ACLK] [get_bd_pins clk_wiz_0/clk_out4]

connect_bd_intf_net [get_bd_intf_pins zynq_ultra_ps_e_0/M_AXI_HPM0_LPD] -boundary_type upper [get_bd_intf_pins axi_interconnect_0/S00_AXI]
connect_bd_intf_net -boundary_type upper [get_bd_intf_pins axi_interconnect_0/M00_AXI] [get_bd_intf_pins axi_intc_0/s_axi]

#rename interconnect
set_property name ps8_0_axi_periph [get_bd_cells axi_interconnect_0]

#add zynqUS interrupt inputs and connect intr IP core
startgroup
set_property -dict [list CONFIG.PSU__USE__IRQ0 {1}] [get_bd_cells zynq_ultra_ps_e_0]
endgroup
connect_bd_net [get_bd_pins axi_intc_0/irq] [get_bd_pins zynq_ultra_ps_e_0/pl_ps_irq0]

# add interrputs to platform
set_property PFM.IRQ {intr { id 0 range 32 }} [get_bd_cells /axi_intc_0]

# add axi buses to platform
set_property PFM.AXI_PORT {M_AXI_HPM0_FPD {memport "M_AXI_GP" sptag "GP0" memory "" is_range "false"} M_AXI_HPM1_FPD {memport "M_AXI_GP" sptag "GP1" memory "" is_range "false"} S_AXI_HPC0_FPD {memport "S_AXI_HP" sptag "HPC0" memory "" is_range "false"} S_AXI_HPC1_FPD {memport "S_AXI_HP" sptag "HPC1" memory "" is_range "false"} S_AXI_HP0_FPD {memport "S_AXI_HP" sptag "HP0" memory "" is_range "false"} S_AXI_HP1_FPD {memport "S_AXI_HP" sptag "HP1" memory "" is_range "false"} S_AXI_HP2_FPD {memport "S_AXI_HP" sptag "HP2" memory "" is_range "false"} S_AXI_HP3_FPD {memport "S_AXI_HP" sptag "HP3" memory "" is_range "false"}} [get_bd_cells /zynq_ultra_ps_e_0]

#add interconnect ports to platform
set_property PFM.AXI_PORT {M01_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M02_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M03_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M04_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M05_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M06_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M07_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"}} [get_bd_cells /ps8_0_axi_periph]


#**********************************************
# VCU begin

#release HP0 and HPC0 for VCU
set_property PFM.AXI_PORT {M_AXI_HPM0_FPD {memport "M_AXI_GP" sptag "GP0" memory "" is_range "false"} M_AXI_HPM1_FPD {memport "M_AXI_GP" sptag "GP1" memory "" is_range "false"} S_AXI_HPC1_FPD {memport "S_AXI_HP" sptag "HPC1" memory "" is_range "false"} S_AXI_HP1_FPD {memport "S_AXI_HP" sptag "HP1" memory "" is_range "false"} S_AXI_HP2_FPD {memport "S_AXI_HP" sptag "HP2" memory "" is_range "false"} S_AXI_HP3_FPD {memport "S_AXI_HP" sptag "HP3" memory "" is_range "false"}} [get_bd_cells /zynq_ultra_ps_e_0]

#release M01_AXI
set_property PFM.AXI_PORT {M02_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M03_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M04_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M05_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M06_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M07_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"}} [get_bd_cells /ps8_0_axi_periph]

#configure clock wizard to generate reference clock for VCU
set_property -dict [list \
  CONFIG.CLKOUT5_USED {true} \
  CONFIG.CLKOUT5_REQUESTED_OUT_FREQ {50.000} \
  CONFIG.MMCM_CLKOUT4_DIVIDE {24} \
  CONFIG.NUM_OUT_CLKS {5} \
  CONFIG.CLKOUT5_JITTER {132.683} \
  CONFIG.CLKOUT5_PHASE_ERROR {87.180} \
  CONFIG.CLK_OUT5_PORT {clk_vcu_pll} \
] [get_bd_cells clk_wiz_0]

# add vcu_0
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:vcu:1.2 vcu_0
endgroup

# vcu config to provide a decoder only
set_property -dict [ list \
  CONFIG.ENABLE_ENCODER {false} \
  CONFIG.ENC_BUFFER_EN {false} \
] [get_bd_cells vcu_0]

# Add SMCs for vcu
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect:1.0 axi_smc_0
create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect:1.0 axi_smc_1
endgroup

#smc config
set_property -dict [ list CONFIG.NUM_SI {1}] [get_bd_cells axi_smc_1]

#add IRQs concat
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat:2.1 IRQs
endgroup

set_property -dict [ list \
  CONFIG.NUM_PORTS {8} \
] [get_bd_cells IRQs]


#enable HP0 and HPC0
startgroup
set_property -dict [list CONFIG.PSU__USE__S_AXI_GP0 {0} CONFIG.PSU__USE__S_AXI_GP2 {1}] [get_bd_cells zynq_ultra_ps_e_0]
set_property -dict [list CONFIG.PSU__USE__S_AXI_GP0 {1}] [get_bd_cells zynq_ultra_ps_e_0]
endgroup

#VCU connections
connect_bd_intf_net -intf_net vcu_0_M_AXI_DEC0 [get_bd_intf_pins axi_smc_0/S00_AXI] [get_bd_intf_pins vcu_0/M_AXI_DEC0]
connect_bd_intf_net -intf_net vcu_0_M_AXI_DEC1 [get_bd_intf_pins axi_smc_0/S01_AXI] [get_bd_intf_pins vcu_0/M_AXI_DEC1]
connect_bd_intf_net -intf_net vcu_0_M_AXI_MCU [get_bd_intf_pins axi_smc_1/S00_AXI] [get_bd_intf_pins vcu_0/M_AXI_MCU]
connect_bd_intf_net -intf_net axi_smc_0_M00_AXI [get_bd_intf_pins axi_smc_0/M00_AXI] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HP0_FPD]
connect_bd_intf_net -intf_net axi_smc_1_M00_AXI [get_bd_intf_pins axi_smc_1/M00_AXI] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HPC0_FPD]

connect_bd_net [get_bd_pins axi_smc_0/aresetn] [get_bd_pins proc_sys_reset_4/interconnect_aresetn]
connect_bd_net [get_bd_pins axi_smc_1/aresetn] [get_bd_pins proc_sys_reset_4/interconnect_aresetn]
connect_bd_net [get_bd_pins vcu_0/vcu_resetn] [get_bd_pins proc_sys_reset_4/peripheral_aresetn]

connect_bd_net [get_bd_pins axi_smc_0/aclk] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins axi_smc_1/aclk] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins vcu_0/m_axi_mcu_aclk] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins vcu_0/m_axi_dec_aclk] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins vcu_0/pll_ref_clk] [get_bd_pins clk_wiz_0/clk_vcu_pll]
connect_bd_net [get_bd_pins vcu_0/s_axi_lite_aclk] [get_bd_pins clk_wiz_0/clk_out1]
connect_bd_net [get_bd_pins zynq_ultra_ps_e_0/saxihpc0_fpd_aclk] [get_bd_pins clk_wiz_0/clk_out4]
connect_bd_net [get_bd_pins zynq_ultra_ps_e_0/saxihp0_fpd_aclk] [get_bd_pins clk_wiz_0/clk_out4]

#delete_bd_objs [get_bd_nets axi_intc_0_irq]
disconnect_bd_net /axi_intc_0_irq [get_bd_pins zynq_ultra_ps_e_0/pl_ps_irq0]
connect_bd_net [get_bd_pins IRQs/In0] [get_bd_pins vcu_0/vcu_host_interrupt]
connect_bd_net [get_bd_pins IRQs/In1] [get_bd_pins axi_intc_0/irq]
connect_bd_net [get_bd_pins IRQs/Dout] [get_bd_pins zynq_ultra_ps_e_0/pl_ps_irq0]

startgroup
set_property -dict [list CONFIG.NUM_MI {2}] [get_bd_cells ps8_0_axi_periph]
endgroup
connect_bd_net [get_bd_pins ps8_0_axi_periph/M01_ACLK] [get_bd_pins clk_wiz_0/clk_out1]
connect_bd_net [get_bd_pins ps8_0_axi_periph/M01_ARESETN] [get_bd_pins proc_sys_reset_1/peripheral_aresetn]

connect_bd_intf_net -boundary_type upper [get_bd_intf_pins ps8_0_axi_periph/M01_AXI] [get_bd_intf_pins vcu_0/S_AXI_LITE]

#add constraint
set fo [open ../constraints/_i_io.xdc a]
puts $fo "\n"
puts $fo "set_clock_groups -asynchronous -group \[get_clocks clk_out4_zusys_clk_wiz_0_0\] -group \[get_clocks clk_out1_zusys_clk_wiz_0_0\]"
close $fo

# VCU end
#**********************************************


#**********************************************
# VDMAs start

#release HP1 for VDMAs
set_property PFM.AXI_PORT {M_AXI_HPM0_FPD {memport "M_AXI_GP" sptag "GP0" memory "" is_range "false"} M_AXI_HPM1_FPD {memport "M_AXI_GP" sptag "GP1" memory "" is_range "false"} S_AXI_HPC1_FPD {memport "S_AXI_HP" sptag "HPC1" memory "" is_range "false"} S_AXI_HP2_FPD {memport "S_AXI_HP" sptag "HP2" memory "" is_range "false"} S_AXI_HP3_FPD {memport "S_AXI_HP" sptag "HP3" memory "" is_range "false"}} [get_bd_cells /zynq_ultra_ps_e_0]

#Set M0x_AXIs
set_property PFM.AXI_PORT {M12_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M13_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M14_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M15_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M16_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M17_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M18_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"} M19_AXI {memport "M_AXI_GP" sptag "" memory "" is_range "false"}} [get_bd_cells /ps8_0_axi_periph]


#---------------------------------------------
# hdmi_out_te0701 hierarchy
create_bd_cell -type hier hdmi_out_te0701

# Create interface pins
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 hdmi_out_te0701/S_AXIS
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:hdmi_rtl:1.0 hdmi_out_te0701/hdmio

# Create pins
  create_bd_pin -dir I -type clk hdmi_out_te0701/aclk
  create_bd_pin -dir I -type rst hdmi_out_te0701/aresetn
  create_bd_pin -dir O -from 0 -to 0 hdmi_out_te0701/hdmio_cec_clk
  create_bd_pin -dir O -from 0 -to 0 hdmi_out_te0701/hdmio_ls_oe
  create_bd_pin -dir O -from 0 -to 0 hdmi_out_te0701/hdmio_ct_hpd
  create_bd_pin -dir I -type clk hdmi_out_te0701/vid_io_out_clk

# Create instance: axis_subset_converter_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter:1.1 hdmi_out_te0701/axis_subset_converter_0
  endgroup

  set_property -dict [ list \
    CONFIG.M_TDATA_NUM_BYTES {3} \
    CONFIG.S_TDATA_NUM_BYTES {4} \
    CONFIG.TDATA_REMAP {tdata[7:0],tdata[23:16],tdata[15:8]} \
  ] [get_bd_cells hdmi_out_te0701/axis_subset_converter_0]

# Create instance: gnd, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 hdmi_out_te0701/gnd
  endgroup

  set_property -dict [ list \
    CONFIG.CONST_VAL {0} \
  ] [get_bd_cells hdmi_out_te0701/gnd]

# Create instance: vcc, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 hdmi_out_te0701/vcc
  endgroup

  set_property -dict [ list \
    CONFIG.CONST_VAL {1} \
  ] [get_bd_cells hdmi_out_te0701/vcc]

# Create instance: v_axi4s_vid_out_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_axi4s_vid_out:4.0 hdmi_out_te0701/v_axi4s_vid_out_0
  endgroup

  set_property -dict [ list \
    CONFIG.C_HAS_ASYNC_CLK {1} \
    CONFIG.C_VTG_MASTER_SLAVE {1} \
  ] [get_bd_cells hdmi_out_te0701/v_axi4s_vid_out_0]

# Create instance: v_tc_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_tc:6.2 hdmi_out_te0701/v_tc_0
  endgroup

  set_property -dict [ list \
    CONFIG.GEN_F0_VBLANK_HEND {960} \
    CONFIG.GEN_F0_VBLANK_HSTART {960} \
    CONFIG.GEN_F0_VFRAME_SIZE {1125} \
    CONFIG.GEN_F0_VSYNC_HEND {1004} \
    CONFIG.GEN_F0_VSYNC_HSTART {1004} \
    CONFIG.GEN_F0_VSYNC_VEND {1088} \
    CONFIG.GEN_F0_VSYNC_VSTART {1083} \
    CONFIG.GEN_F1_VBLANK_HEND {960} \
    CONFIG.GEN_F1_VBLANK_HSTART {960} \
    CONFIG.GEN_F1_VFRAME_SIZE {1125} \
    CONFIG.GEN_F1_VSYNC_HEND {1004} \
    CONFIG.GEN_F1_VSYNC_HSTART {1004} \
    CONFIG.GEN_F1_VSYNC_VEND {1088} \
    CONFIG.GEN_F1_VSYNC_VSTART {1083} \
    CONFIG.GEN_HACTIVE_SIZE {1920} \
    CONFIG.GEN_HFRAME_SIZE {2200} \
    CONFIG.GEN_HSYNC_END {2052} \
    CONFIG.GEN_HSYNC_START {2008} \
    CONFIG.GEN_VACTIVE_SIZE {1080} \
    CONFIG.HAS_AXI4_LITE {false} \
    CONFIG.VIDEO_MODE {1080p} \
    CONFIG.enable_detection {false} \
 ] [get_bd_cells hdmi_out_te0701/v_tc_0]

# Create instance: video_io_to_hdmi_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv trenz.biz:user:video_io_to_hdmi:1.1 hdmi_out_te0701/video_io_to_hdmi_0
  endgroup

# Create interface connections
  connect_bd_intf_net -intf_net axi_vdma_0_M_AXIS_MM2S          [get_bd_intf_pins hdmi_out_te0701/S_AXIS]                         [get_bd_intf_pins hdmi_out_te0701/axis_subset_converter_0/S_AXIS]
  connect_bd_intf_net -intf_net axis_subset_converter_0_M_AXIS  [get_bd_intf_pins hdmi_out_te0701/axis_subset_converter_0/M_AXIS] [get_bd_intf_pins hdmi_out_te0701/v_axi4s_vid_out_0/video_in]
  connect_bd_intf_net -intf_net v_axi4s_vid_out_0_vid_io_out    [get_bd_intf_pins hdmi_out_te0701/v_axi4s_vid_out_0/vid_io_out]   [get_bd_intf_pins hdmi_out_te0701/video_io_to_hdmi_0/vid_io_in]
  connect_bd_intf_net -intf_net v_tc_0_vtiming_out              [get_bd_intf_pins hdmi_out_te0701/v_axi4s_vid_out_0/vtiming_in]   [get_bd_intf_pins hdmi_out_te0701/v_tc_0/vtiming_out]
  connect_bd_intf_net -intf_net video_io_to_hdmi_0_hdmi_out     [get_bd_intf_pins hdmi_out_te0701/hdmio]                          [get_bd_intf_pins hdmi_out_te0701/video_io_to_hdmi_0/hdmi_out]

# Create port connections
  connect_bd_net -net clk_wiz_0_clk_out4                  [get_bd_pins hdmi_out_te0701/aclk]            [get_bd_pins hdmi_out_te0701/axis_subset_converter_0/aclk] [get_bd_pins hdmi_out_te0701/v_axi4s_vid_out_0/aclk]
  connect_bd_net -net clk_wiz_1_clk_148_5_MHz             [get_bd_pins hdmi_out_te0701/vid_io_out_clk]  [get_bd_pins hdmi_out_te0701/v_axi4s_vid_out_0/vid_io_out_clk] [get_bd_pins hdmi_out_te0701/v_tc_0/clk] [get_bd_pins hdmi_out_te0701/video_io_to_hdmi_0/vid_io_in_clk]
  connect_bd_net -net gnd_dout                            [get_bd_pins hdmi_out_te0701/hdmio_cec_clk]   [get_bd_pins hdmi_out_te0701/gnd/dout]
  connect_bd_net -net proc_sys_reset_4_peripheral_aresetn [get_bd_pins hdmi_out_te0701/aresetn]         [get_bd_pins hdmi_out_te0701/axis_subset_converter_0/aresetn] [get_bd_pins hdmi_out_te0701/v_axi4s_vid_out_0/aresetn]
  connect_bd_net -net vcc_dout                            [get_bd_pins hdmi_out_te0701/vcc/dout]        [get_bd_pins hdmi_out_te0701/hdmio_ls_oe] [get_bd_pins hdmi_out_te0701/hdmio_ct_hpd]

  regenerate_bd_layout -hierarchy [get_bd_cells hdmi_out_te0701]


#---------------------------------------------
# hdmi_in hierarchy
create_bd_cell -type hier hdmi_in

# Create interface pins
  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 hdmi_in/AXIS_vid_out
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 hdmi_in/AXI_lite_hb_det
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 hdmi_in/AXI_lite_sig_det
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 hdmi_in/AXI_lite_v_proc

# Create pins
  create_bd_pin -dir I hdmi_in/axi_lite_clk
  create_bd_pin -dir I hdmi_in/axi_lite_resetn
  create_bd_pin -dir I hdmi_in/axis_clk
  create_bd_pin -dir I hdmi_in/axis_resetn
  create_bd_pin -dir I -from 15 -to 0 hdmi_in/hdmi_vid_in
  create_bd_pin -dir I hdmi_in/hdmi_vid_in_clk

# Create instance: im_hdmi_in_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv UTIA:utia:im_hdmi_in:1.0 hdmi_in/im_hdmi_in_0
  endgroup

# Create instance: v_vid_in_axi4s_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_vid_in_axi4s:5.0 hdmi_in/v_vid_in_axi4s_0
  endgroup

  set_property -dict [ list \
    CONFIG.C_ADDR_WIDTH {12} \
    CONFIG.C_HAS_ASYNC_CLK {1} \
    CONFIG.C_M_AXIS_VIDEO_FORMAT {0} \
  ] [get_bd_cells hdmi_in/v_vid_in_axi4s_0]

# Create instance: axis_vid_det_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv UTIA:utia:axis_vid_det:1.0 hdmi_in/axis_vid_det_0
  endgroup

  set_property -dict [ list \
    CONFIG.C_TDATA_WIDTH {16} \
    CONFIG.C_X_CNT {3840} \
    CONFIG.C_Y_CNT {2160} \
  ] [get_bd_cells hdmi_in/axis_vid_det_0]

# Create instance: v_proc_ss_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_proc_ss:2.3 hdmi_in/v_proc_ss_0
  endgroup

  set_property -dict [ list \
    CONFIG.C_AXIMM_DATA_WIDTH {64} \
    CONFIG.C_COLORSPACE_SUPPORT {1} \
    CONFIG.C_CSC_ENABLE_WINDOW {false} \
    CONFIG.C_MAX_DATA_WIDTH {8} \
    CONFIG.C_SAMPLES_PER_CLK {1} \
    CONFIG.C_TOPOLOGY {3} \
  ] [get_bd_cells hdmi_in/v_proc_ss_0]

# Create instance: RBG_to_BGR, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter:1.1 hdmi_in/RBG_to_BGR
  endgroup

  set_property -dict [ list \
   CONFIG.M_TDATA_NUM_BYTES {3} \
   CONFIG.S_TDATA_NUM_BYTES {3} \
   CONFIG.TDATA_REMAP {tdata[23:16],tdata[7:0],tdata[15:8]} \
 ] [get_bd_cells hdmi_in/RBG_to_BGR]

# Create instance: hblank_det_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv UTIA:utia:hblank_det:1.0 hdmi_in/hblank_det_0
  endgroup

# Create interface connections
  connect_bd_intf_net -intf_net AXI_lite_sig_det_1          [get_bd_intf_pins hdmi_in/AXI_lite_sig_det]             [get_bd_intf_pins hdmi_in/axis_vid_det_0/S00_AXI]
  connect_bd_intf_net -intf_net AXI_lite_v_proc_1           [get_bd_intf_pins hdmi_in/AXI_lite_v_proc]              [get_bd_intf_pins hdmi_in/v_proc_ss_0/s_axi_ctrl]
  connect_bd_intf_net -intf_net RBG_to_BGR_M_AXIS           [get_bd_intf_pins hdmi_in/AXIS_vid_out]                 [get_bd_intf_pins hdmi_in/RBG_to_BGR/M_AXIS]
  connect_bd_intf_net -intf_net axi_lite_hb_det_1           [get_bd_intf_pins hdmi_in/AXI_lite_hb_det]              [get_bd_intf_pins hdmi_in/hblank_det_0/S00_AXI]
  connect_bd_intf_net -intf_net axis_vid_det_0_AXIS_VID_OUT [get_bd_intf_pins hdmi_in/axis_vid_det_0/AXIS_VID_OUT]  [get_bd_intf_pins hdmi_in/v_proc_ss_0/s_axis]
  connect_bd_intf_net -intf_net im_hdmi_in_0_vid_out        [get_bd_intf_pins hdmi_in/im_hdmi_in_0/vid_out]         [get_bd_intf_pins hdmi_in/v_vid_in_axi4s_0/vid_io_in]
  connect_bd_intf_net -intf_net v_proc_ss_0_m_axis          [get_bd_intf_pins hdmi_in/RBG_to_BGR/S_AXIS]            [get_bd_intf_pins hdmi_in/v_proc_ss_0/m_axis]
  connect_bd_intf_net -intf_net v_vid_in_axi4s_0_video_out  [get_bd_intf_pins hdmi_in/axis_vid_det_0/AXIS_VID_IN]   [get_bd_intf_pins hdmi_in/v_vid_in_axi4s_0/video_out]

# Create port connections
  connect_bd_net -net axi_lite_clk_1          [get_bd_pins hdmi_in/axi_lite_clk]        [get_bd_pins hdmi_in/axis_vid_det_0/s00_axi_aclk] [get_bd_pins hdmi_in/hblank_det_0/s00_axi_aclk]
  connect_bd_net -net axi_lite_resetn_1       [get_bd_pins hdmi_in/axi_lite_resetn]     [get_bd_pins hdmi_in/axis_vid_det_0/s00_axi_aresetn] [get_bd_pins hdmi_in/hblank_det_0/s00_axi_aresetn]
  connect_bd_net -net axis_clk_1              [get_bd_pins hdmi_in/axis_clk]            [get_bd_pins hdmi_in/RBG_to_BGR/aclk] [get_bd_pins hdmi_in/axis_vid_det_0/aclk] [get_bd_pins hdmi_in/hblank_det_0/clk] [get_bd_pins hdmi_in/v_proc_ss_0/aclk] [get_bd_pins hdmi_in/v_vid_in_axi4s_0/aclk]
  connect_bd_net -net axis_resetn_1           [get_bd_pins hdmi_in/axis_resetn]         [get_bd_pins hdmi_in/RBG_to_BGR/aresetn] [get_bd_pins hdmi_in/axis_vid_det_0/aresetn] [get_bd_pins hdmi_in/hblank_det_0/rstn] [get_bd_pins hdmi_in/v_proc_ss_0/aresetn] [get_bd_pins hdmi_in/v_vid_in_axi4s_0/aresetn]
  connect_bd_net -net hdmi_vid_in_1           [get_bd_pins hdmi_in/hdmi_vid_in]         [get_bd_pins hdmi_in/im_hdmi_in_0/hdmi_data]
  connect_bd_net -net hdmi_vid_in_clk_1       [get_bd_pins hdmi_in/hdmi_vid_in_clk]     [get_bd_pins hdmi_in/im_hdmi_in_0/hdmi_clk] [get_bd_pins hdmi_in/v_vid_in_axi4s_0/vid_io_in_clk]
  connect_bd_net -net im_hdmi_in_0_vid_hblank [get_bd_pins hdmi_in/hblank_det_0/hblank] [get_bd_pins hdmi_in/im_hdmi_in_0/vid_hblank]

  regenerate_bd_layout -hierarchy [get_bd_cells hdmi_in]


#---------------------------------------------
# hdmi_out hierarchy
create_bd_cell -type hier hdmi_out

# Create interface pins
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 hdmi_out/AXIS_vid_in
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 hdmi_out/AXI_lite_v_proc
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:aximm_rtl:1.0 hdmi_out/AXI_lite_vtc

 # Create pins
  create_bd_pin -dir I hdmi_out/axi_lite_clk
  create_bd_pin -dir I hdmi_out/axi_lite_resetn
  create_bd_pin -dir I hdmi_out/axis_clk
  create_bd_pin -dir I hdmi_out/axis_resetn
  create_bd_pin -dir O -from 15 -to 0 hdmi_out/hdmi_vid_out
  create_bd_pin -dir O hdmi_out/hdmi_vid_out_clk
  create_bd_pin -dir O hdmi_out/irq
  create_bd_pin -dir I hdmi_out/vid_out_clk

# Create instance: BGR_to_RBG, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter:1.1 hdmi_out/BGR_to_RBG
  endgroup

  set_property -dict [ list \
    CONFIG.M_TDATA_NUM_BYTES {3} \
    CONFIG.S_TDATA_NUM_BYTES {3} \
    CONFIG.TDATA_REMAP {tdata[23:16],tdata[7:0],tdata[15:8]} \
  ] [get_bd_cells hdmi_out/BGR_to_RBG]

# Create instance: axis_subset_converter_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter:1.1 hdmi_out/axis_subset_converter_0
  endgroup

  set_property -dict [ list \
    CONFIG.M_TDATA_NUM_BYTES {2} \
    CONFIG.S_TDATA_NUM_BYTES {3} \
    CONFIG.TDATA_REMAP {tdata[15:0]} \
  ] [get_bd_cells hdmi_out/axis_subset_converter_0]

# Create instance: im_hdmi_out_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv UTIA:utia:im_hdmi_out:1.2 hdmi_out/im_hdmi_out_0
  endgroup

# Create instance: v_axi4s_vid_out_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_axi4s_vid_out:4.0 hdmi_out/v_axi4s_vid_out_0
  endgroup

  set_property -dict [ list \
    CONFIG.C_ADDR_WIDTH {12} \
    CONFIG.C_HAS_ASYNC_CLK {1} \
    CONFIG.C_S_AXIS_VIDEO_FORMAT {0} \
    CONFIG.C_VTG_MASTER_SLAVE {1} \
  ] [get_bd_cells hdmi_out/v_axi4s_vid_out_0]

# Create instance: v_proc_ss_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_proc_ss:2.3 hdmi_out/v_proc_ss_0
  endgroup

  set_property -dict [ list \
    CONFIG.C_AXIMM_DATA_WIDTH {64} \
    CONFIG.C_COLORSPACE_SUPPORT {1} \
    CONFIG.C_CSC_ENABLE_WINDOW {false} \
    CONFIG.C_MAX_DATA_WIDTH {8} \
    CONFIG.C_SAMPLES_PER_CLK {1} \
    CONFIG.C_TOPOLOGY {3} \
  ] [get_bd_cells hdmi_out/v_proc_ss_0]

# Create instance: v_tc_0, and set properties
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_tc:6.2 hdmi_out/v_tc_0
  endgroup

  set_property -dict [ list \
    CONFIG.GEN_F0_VBLANK_HEND {960} \
    CONFIG.GEN_F0_VBLANK_HSTART {960} \
    CONFIG.GEN_F0_VFRAME_SIZE {1125} \
    CONFIG.GEN_F0_VSYNC_HEND {1004} \
    CONFIG.GEN_F0_VSYNC_HSTART {1004} \
    CONFIG.GEN_F0_VSYNC_VEND {1088} \
    CONFIG.GEN_F0_VSYNC_VSTART {1083} \
    CONFIG.GEN_F1_VBLANK_HEND {960} \
    CONFIG.GEN_F1_VBLANK_HSTART {960} \
    CONFIG.GEN_F1_VFRAME_SIZE {1125} \
    CONFIG.GEN_F1_VSYNC_HEND {1004} \
    CONFIG.GEN_F1_VSYNC_HSTART {1004} \
    CONFIG.GEN_F1_VSYNC_VEND {1088} \
    CONFIG.GEN_F1_VSYNC_VSTART {1083} \
    CONFIG.GEN_HACTIVE_SIZE {1920} \
    CONFIG.GEN_HFRAME_SIZE {2200} \
    CONFIG.GEN_HSYNC_END {2052} \
    CONFIG.GEN_HSYNC_START {2008} \
    CONFIG.GEN_VACTIVE_SIZE {1080} \
    CONFIG.VIDEO_MODE {1080p} \
    CONFIG.enable_detection {false} \
  ] [get_bd_cells hdmi_out/v_tc_0]

# Create interface connections
  connect_bd_intf_net -intf_net AXIS_vid_in_1                   [get_bd_intf_pins hdmi_out/AXIS_vid_in]                     [get_bd_intf_pins hdmi_out/BGR_to_RBG/S_AXIS]
  connect_bd_intf_net -intf_net AXI_lite_v_proc_1               [get_bd_intf_pins hdmi_out/AXI_lite_v_proc]                 [get_bd_intf_pins hdmi_out/v_proc_ss_0/s_axi_ctrl]
  connect_bd_intf_net -intf_net AXI_lite_vtc_1                  [get_bd_intf_pins hdmi_out/AXI_lite_vtc]                    [get_bd_intf_pins hdmi_out/v_tc_0/ctrl]
  connect_bd_intf_net -intf_net BGR_to_RBG_M_AXIS               [get_bd_intf_pins hdmi_out/BGR_to_RBG/M_AXIS]               [get_bd_intf_pins hdmi_out/v_proc_ss_0/s_axis]
  connect_bd_intf_net -intf_net axis_subset_converter_0_M_AXIS  [get_bd_intf_pins hdmi_out/axis_subset_converter_0/M_AXIS]  [get_bd_intf_pins hdmi_out/v_axi4s_vid_out_0/video_in]
  connect_bd_intf_net -intf_net v_axi4s_vid_out_0_vid_io_out    [get_bd_intf_pins hdmi_out/im_hdmi_out_0/vid_in]            [get_bd_intf_pins hdmi_out/v_axi4s_vid_out_0/vid_io_out]
  connect_bd_intf_net -intf_net v_proc_ss_0_m_axis              [get_bd_intf_pins hdmi_out/axis_subset_converter_0/S_AXIS]  [get_bd_intf_pins hdmi_out/v_proc_ss_0/m_axis]
  connect_bd_intf_net -intf_net v_tc_0_vtiming_out              [get_bd_intf_pins hdmi_out/v_axi4s_vid_out_0/vtiming_in]    [get_bd_intf_pins hdmi_out/v_tc_0/vtiming_out]

# Create port connections
  connect_bd_net -net axi_lite_clk_1            [get_bd_pins hdmi_out/axi_lite_clk]     [get_bd_pins hdmi_out/v_tc_0/s_axi_aclk]
  connect_bd_net -net axi_lite_resetn_1         [get_bd_pins hdmi_out/axi_lite_resetn]  [get_bd_pins hdmi_out/v_tc_0/s_axi_aresetn]
  connect_bd_net -net axis_clk_1                [get_bd_pins hdmi_out/axis_clk]         [get_bd_pins hdmi_out/BGR_to_RBG/aclk] [get_bd_pins hdmi_out/axis_subset_converter_0/aclk] [get_bd_pins hdmi_out/v_axi4s_vid_out_0/aclk] [get_bd_pins hdmi_out/v_proc_ss_0/aclk]
  connect_bd_net -net axis_resetn_1             [get_bd_pins hdmi_out/axis_resetn]      [get_bd_pins hdmi_out/BGR_to_RBG/aresetn] [get_bd_pins hdmi_out/axis_subset_converter_0/aresetn] [get_bd_pins hdmi_out/v_axi4s_vid_out_0/aresetn] [get_bd_pins hdmi_out/v_proc_ss_0/aresetn]
  connect_bd_net -net im_hdmi_out_0_hdmi_clk    [get_bd_pins hdmi_out/hdmi_vid_out_clk] [get_bd_pins hdmi_out/im_hdmi_out_0/hdmi_clk]
  connect_bd_net -net im_hdmi_out_0_hdmi_data   [get_bd_pins hdmi_out/hdmi_vid_out]     [get_bd_pins hdmi_out/im_hdmi_out_0/hdmi_data]
  connect_bd_net -net v_axi4s_vid_out_0_locked  [get_bd_pins hdmi_out/im_hdmi_out_0/oe] [get_bd_pins hdmi_out/v_axi4s_vid_out_0/locked]
  connect_bd_net -net v_tc_0_irq                [get_bd_pins hdmi_out/irq]              [get_bd_pins hdmi_out/v_tc_0/irq]
  connect_bd_net -net vid_out_clk_1             [get_bd_pins hdmi_out/vid_out_clk]      [get_bd_pins hdmi_out/im_hdmi_out_0/vid_clk] [get_bd_pins hdmi_out/v_axi4s_vid_out_0/vid_io_out_clk] [get_bd_pins hdmi_out/v_tc_0/clk]

  regenerate_bd_layout -hierarchy [get_bd_cells hdmi_out]



# Create external ports
create_bd_intf_port -mode Master -vlnv xilinx.com:interface:hdmi_rtl:1.0 hdmioo
create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 hdmioo_i2c
create_bd_intf_port -mode Master -vlnv xilinx.com:interface:iic_rtl:1.0 im_i2c
create_bd_port -dir I -from 15 -to 0 hdmii
create_bd_port -dir I hdmii_clk
create_bd_port -dir O -from 15 -to 0 hdmio
create_bd_port -dir O hdmio_clk
create_bd_port -dir O -from 0 -to 0 hdmioo_cec_clk
create_bd_port -dir O -from 0 -to 0 hdmioo_ct_hpd
create_bd_port -dir O -from 0 -to 0 hdmioo_ls_oe
create_bd_port -dir O -from 0 -to 0 im_resetn


# Create instance: axi_iic_0, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_iic:2.1 axi_iic_0
endgroup
set_property -dict [ list \
  CONFIG.C_SCL_INERTIAL_DELAY {5} \
  CONFIG.C_SDA_INERTIAL_DELAY {5} \
] [get_bd_cells axi_iic_0]

# Create instance: axi_iic_1, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_iic:2.1 axi_iic_1
endgroup
set_property -dict [ list \
  CONFIG.C_SCL_INERTIAL_DELAY {5} \
  CONFIG.C_SDA_INERTIAL_DELAY {5} \
] [get_bd_cells axi_iic_1]

# Create instance: axi_smc_2, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:smartconnect:1.0 axi_smc_2
endgroup
set_property -dict [ list \
  CONFIG.NUM_SI {3} \
] [get_bd_cells axi_smc_2]

# Create instance: axi_vdma_0, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_vdma:6.3 axi_vdma_0
endgroup
set_property -dict [ list \
  CONFIG.c_include_s2mm {0} \
  CONFIG.c_m_axi_mm2s_data_width {128} \
  CONFIG.c_mm2s_genlock_mode {0} \
  CONFIG.c_mm2s_linebuffer_depth {2048} \
  CONFIG.c_mm2s_max_burst_length {8} \
  CONFIG.c_num_fstores {1} \
  CONFIG.c_s2mm_genlock_mode {0} \
] [get_bd_cells axi_vdma_0]

# Create instance: axi_vdma_1, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_vdma:6.3 axi_vdma_1
endgroup
set_property -dict [ list \
  CONFIG.c_m_axi_mm2s_data_width {128} \
  CONFIG.c_m_axi_s2mm_data_width {128} \
  CONFIG.c_m_axis_mm2s_tdata_width {24} \
  CONFIG.c_mm2s_linebuffer_depth {4096} \
  CONFIG.c_mm2s_max_burst_length {32} \
  CONFIG.c_num_fstores {7} \
  CONFIG.c_s2mm_linebuffer_depth {4096} \
  CONFIG.c_s2mm_max_burst_length {32} \
] [get_bd_cells axi_vdma_1]

# Create instance: clk_wiz_1, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 clk_wiz_1
endgroup
set_property -dict [ list \
  CONFIG.CLKOUT1_JITTER {228.536} \
  CONFIG.CLKOUT1_PHASE_ERROR {394.762} \
  CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {148.5} \
  CONFIG.CLK_OUT1_PORT {clk_148_5_MHz} \
  CONFIG.MMCM_CLKFBOUT_MULT_F {111.375} \
  CONFIG.MMCM_CLKOUT0_DIVIDE_F {7.500} \
  CONFIG.MMCM_DIVCLK_DIVIDE {10} \
  CONFIG.RESET_PORT {resetn} \
  CONFIG.RESET_TYPE {ACTIVE_LOW} \
] [get_bd_cells clk_wiz_1]

# Create instance: clk_wiz_2, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 clk_wiz_2
endgroup
set_property -dict [ list \
  CONFIG.CLKOUT1_DRIVES {Buffer} \
  CONFIG.CLKOUT1_JITTER {228.536} \
  CONFIG.CLKOUT1_PHASE_ERROR {394.762} \
  CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {148.500} \
  CONFIG.CLKOUT2_DRIVES {Buffer} \
  CONFIG.CLKOUT2_JITTER {109.471} \
  CONFIG.CLKOUT2_PHASE_ERROR {82.897} \
  CONFIG.CLKOUT2_USED {false} \
  CONFIG.CLKOUT3_DRIVES {Buffer} \
  CONFIG.CLKOUT4_DRIVES {Buffer} \
  CONFIG.CLKOUT5_DRIVES {Buffer} \
  CONFIG.CLKOUT6_DRIVES {Buffer} \
  CONFIG.CLKOUT7_DRIVES {Buffer} \
  CONFIG.CLK_OUT1_PORT {clk_prog} \
  CONFIG.MMCM_CLKFBOUT_MULT_F {111.375} \
  CONFIG.MMCM_CLKOUT0_DIVIDE_F {7.500} \
  CONFIG.MMCM_CLKOUT1_DIVIDE {1} \
  CONFIG.MMCM_DIVCLK_DIVIDE {10} \
  CONFIG.NUM_OUT_CLKS {1} \
  CONFIG.SECONDARY_SOURCE {Single_ended_clock_capable_pin} \
  CONFIG.USE_DYN_RECONFIG {true} \
  CONFIG.USE_PHASE_ALIGNMENT {true} \
] [get_bd_cells clk_wiz_2]

#enable HP1 vor VDMAs
startgroup
set_property -dict [list CONFIG.PSU__USE__S_AXI_GP3 {1}] [get_bd_cells zynq_ultra_ps_e_0]
endgroup

#incnumber of ps8_0_axi_periph master ports
startgroup
set_property -dict [list CONFIG.NUM_MI {12}] [get_bd_cells ps8_0_axi_periph]
endgroup

# Create instance: util_vector_logic_0, and set properties
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:util_vector_logic:2.0 util_vector_logic_0
endgroup
set_property -dict [ list \
  CONFIG.C_SIZE {1} \
] [get_bd_cells util_vector_logic_0]

# Create interface connections
connect_bd_intf_net -intf_net AXIS_vid_in_1 [get_bd_intf_pins axi_vdma_1/M_AXIS_MM2S] [get_bd_intf_pins hdmi_out/AXIS_vid_in]
connect_bd_intf_net -intf_net AXI_lite_v_proc_1 [get_bd_intf_pins hdmi_out/AXI_lite_v_proc] [get_bd_intf_pins ps8_0_axi_periph/M08_AXI]
connect_bd_intf_net -intf_net AXI_lite_vtc_1 [get_bd_intf_pins hdmi_out/AXI_lite_vtc] [get_bd_intf_pins ps8_0_axi_periph/M09_AXI]
connect_bd_intf_net -intf_net axi_iic_0_IIC [get_bd_intf_ports hdmioo_i2c] [get_bd_intf_pins axi_iic_0/IIC]
connect_bd_intf_net -intf_net axi_iic_1_IIC [get_bd_intf_ports im_i2c] [get_bd_intf_pins axi_iic_1/IIC]
connect_bd_intf_net -intf_net axi_smc_2_M00_AXI [get_bd_intf_pins axi_smc_2/M00_AXI] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HP1_FPD]
connect_bd_intf_net -intf_net axi_vdma_0_M_AXIS_MM2S [get_bd_intf_pins axi_vdma_0/M_AXIS_MM2S] [get_bd_intf_pins hdmi_out_te0701/S_AXIS]
connect_bd_intf_net -intf_net axi_vdma_0_M_AXI_MM2S [get_bd_intf_pins axi_smc_2/S00_AXI] [get_bd_intf_pins axi_vdma_0/M_AXI_MM2S]
connect_bd_intf_net -intf_net axi_vdma_1_M_AXI_MM2S [get_bd_intf_pins axi_smc_2/S01_AXI] [get_bd_intf_pins axi_vdma_1/M_AXI_MM2S]
connect_bd_intf_net -intf_net axi_vdma_1_M_AXI_S2MM [get_bd_intf_pins axi_smc_2/S02_AXI] [get_bd_intf_pins axi_vdma_1/M_AXI_S2MM]
connect_bd_intf_net -intf_net hdmi_in_AXIS_vid_out [get_bd_intf_pins axi_vdma_1/S_AXIS_S2MM] [get_bd_intf_pins hdmi_in/AXIS_vid_out]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M02_AXI [get_bd_intf_pins axi_vdma_0/S_AXI_LITE] [get_bd_intf_pins ps8_0_axi_periph/M02_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M03_AXI [get_bd_intf_pins axi_iic_0/S_AXI] [get_bd_intf_pins ps8_0_axi_periph/M03_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M04_AXI [get_bd_intf_pins hdmi_in/AXI_lite_hb_det] [get_bd_intf_pins ps8_0_axi_periph/M04_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M05_AXI [get_bd_intf_pins hdmi_in/AXI_lite_sig_det] [get_bd_intf_pins ps8_0_axi_periph/M05_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M06_AXI [get_bd_intf_pins hdmi_in/AXI_lite_v_proc] [get_bd_intf_pins ps8_0_axi_periph/M06_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M07_AXI [get_bd_intf_pins axi_vdma_1/S_AXI_LITE] [get_bd_intf_pins ps8_0_axi_periph/M07_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M10_AXI [get_bd_intf_pins axi_iic_1/S_AXI] [get_bd_intf_pins ps8_0_axi_periph/M10_AXI]
connect_bd_intf_net -intf_net ps8_0_axi_periph_M11_AXI [get_bd_intf_pins clk_wiz_2/s_axi_lite] [get_bd_intf_pins ps8_0_axi_periph/M11_AXI]
connect_bd_intf_net -intf_net video_io_to_hdmi_0_hdmi_out [get_bd_intf_ports hdmioo] [get_bd_intf_pins hdmi_out_te0701/hdmio]

# Create port connections
connect_bd_net -net M01_ARESETN_1 [get_bd_pins axi_iic_0/s_axi_aresetn] [get_bd_pins axi_iic_1/s_axi_aresetn] [get_bd_pins axi_vdma_0/axi_resetn] [get_bd_pins axi_vdma_1/axi_resetn] [get_bd_pins clk_wiz_2/s_axi_aresetn] [get_bd_pins hdmi_in/axi_lite_resetn] [get_bd_pins hdmi_out/axi_lite_resetn] [get_bd_pins proc_sys_reset_1/peripheral_aresetn] [get_bd_pins ps8_0_axi_periph/M01_ARESETN] [get_bd_pins ps8_0_axi_periph/M02_ARESETN] [get_bd_pins ps8_0_axi_periph/M03_ARESETN] [get_bd_pins ps8_0_axi_periph/M04_ARESETN] [get_bd_pins ps8_0_axi_periph/M05_ARESETN] [get_bd_pins ps8_0_axi_periph/M07_ARESETN] [get_bd_pins ps8_0_axi_periph/M09_ARESETN] [get_bd_pins ps8_0_axi_periph/M10_ARESETN] [get_bd_pins ps8_0_axi_periph/M11_ARESETN]
connect_bd_net -net S00_ARESETN_1 [get_bd_pins axi_smc_2/aresetn] [get_bd_pins proc_sys_reset_4/interconnect_aresetn]

disconnect_bd_net /axi_intc_0_irq [get_bd_pins IRQs/In1]
connect_bd_net -net axi_vdma_0_mm2s_introut [get_bd_pins IRQs/In1] [get_bd_pins axi_vdma_0/mm2s_introut]
connect_bd_net -net axi_iic_0_iic2intc_irpt [get_bd_pins IRQs/In2] [get_bd_pins axi_iic_0/iic2intc_irpt]
connect_bd_net -net axi_vdma_1_mm2s_introut [get_bd_pins IRQs/In3] [get_bd_pins axi_vdma_1/mm2s_introut]
connect_bd_net -net axi_vdma_1_s2mm_introut [get_bd_pins IRQs/In4] [get_bd_pins axi_vdma_1/s2mm_introut]
connect_bd_net -net hdmi_out_irq            [get_bd_pins IRQs/In5] [get_bd_pins hdmi_out/irq]
connect_bd_net -net axi_iic_1_iic2intc_irpt [get_bd_pins IRQs/In6] [get_bd_pins axi_iic_1/iic2intc_irpt]
connect_bd_net -net axi_intc_0_irq          [get_bd_pins IRQs/In7] [get_bd_pins axi_intc_0/irq]

connect_bd_net -net clk_wiz_0_clk_out1 [get_bd_pins axi_iic_0/s_axi_aclk] [get_bd_pins axi_iic_1/s_axi_aclk] [get_bd_pins axi_vdma_0/s_axi_lite_aclk] [get_bd_pins axi_vdma_1/s_axi_lite_aclk] [get_bd_pins clk_wiz_0/clk_out1] [get_bd_pins clk_wiz_2/s_axi_aclk] [get_bd_pins hdmi_in/axi_lite_clk] [get_bd_pins hdmi_out/axi_lite_clk] [get_bd_pins proc_sys_reset_1/slowest_sync_clk] [get_bd_pins ps8_0_axi_periph/M01_ACLK] [get_bd_pins ps8_0_axi_periph/M02_ACLK] [get_bd_pins ps8_0_axi_periph/M03_ACLK] [get_bd_pins ps8_0_axi_periph/M04_ACLK] [get_bd_pins ps8_0_axi_periph/M05_ACLK] [get_bd_pins ps8_0_axi_periph/M07_ACLK] [get_bd_pins ps8_0_axi_periph/M09_ACLK] [get_bd_pins ps8_0_axi_periph/M10_ACLK] [get_bd_pins ps8_0_axi_periph/M11_ACLK] [get_bd_pins vcu_0/s_axi_lite_aclk]
connect_bd_net -net clk_wiz_0_clk_out4 [get_bd_pins axi_intc_0/s_axi_aclk] [get_bd_pins axi_smc_0/aclk] [get_bd_pins axi_smc_1/aclk] [get_bd_pins axi_smc_2/aclk] [get_bd_pins axi_vdma_0/m_axi_mm2s_aclk] [get_bd_pins axi_vdma_0/m_axis_mm2s_aclk] [get_bd_pins axi_vdma_1/m_axi_mm2s_aclk] [get_bd_pins axi_vdma_1/m_axi_s2mm_aclk] [get_bd_pins axi_vdma_1/m_axis_mm2s_aclk] [get_bd_pins axi_vdma_1/s_axis_s2mm_aclk] [get_bd_pins clk_wiz_0/clk_out4] [get_bd_pins hdmi_in/axis_clk] [get_bd_pins hdmi_out/axis_clk] [get_bd_pins hdmi_out_te0701/aclk] [get_bd_pins proc_sys_reset_4/slowest_sync_clk] [get_bd_pins ps8_0_axi_periph/ACLK] [get_bd_pins ps8_0_axi_periph/M00_ACLK] [get_bd_pins ps8_0_axi_periph/M06_ACLK] [get_bd_pins ps8_0_axi_periph/M08_ACLK] [get_bd_pins ps8_0_axi_periph/S00_ACLK] [get_bd_pins vcu_0/m_axi_dec_aclk] [get_bd_pins vcu_0/m_axi_mcu_aclk] [get_bd_pins zynq_ultra_ps_e_0/maxihpm0_lpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihp0_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihp1_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/saxihpc0_fpd_aclk]

delete_bd_objs [get_bd_nets clk_wiz_0_locked]
connect_bd_net -net clk_wiz_0_locked [get_bd_pins clk_wiz_0/locked] [get_bd_pins util_vector_logic_0/Op1]
connect_bd_net -net clk_wiz_1_locked [get_bd_pins clk_wiz_1/locked] [get_bd_pins util_vector_logic_0/Op2]
connect_bd_net [get_bd_pins util_vector_logic_0/Res] [get_bd_pins proc_sys_reset_1/dcm_locked] [get_bd_pins proc_sys_reset_2/dcm_locked] [get_bd_pins proc_sys_reset_3/dcm_locked] [get_bd_pins proc_sys_reset_4/dcm_locked]

connect_bd_net -net clk_wiz_1_clk_148_5_MHz [get_bd_pins clk_wiz_1/clk_148_5_MHz] [get_bd_pins hdmi_out_te0701/vid_io_out_clk]
connect_bd_net -net clk_wiz_2_clk_prog [get_bd_pins clk_wiz_2/clk_prog] [get_bd_pins hdmi_out/vid_out_clk]

connect_bd_net -net hdmi_out_te0701_cec_clk [get_bd_ports hdmioo_cec_clk] [get_bd_pins hdmi_out_te0701/hdmio_cec_clk]
connect_bd_net -net hdmi_out_te0701_ls_oe   [get_bd_ports hdmioo_ls_oe]   [get_bd_pins hdmi_out_te0701/hdmio_ls_oe]
connect_bd_net -net hdmi_out_te0701_ct_hpd  [get_bd_ports hdmioo_ct_hpd]  [get_bd_pins hdmi_out_te0701/hdmio_ct_hpd]

connect_bd_net -net hdmi_out_hdmi_vid_out [get_bd_ports hdmio] [get_bd_pins hdmi_out/hdmi_vid_out]
connect_bd_net -net hdmi_out_hdmi_vid_out_clk [get_bd_ports hdmio_clk] [get_bd_pins hdmi_out/hdmi_vid_out_clk]

connect_bd_net -net hdmi_vid_in_0_1 [get_bd_ports hdmii] [get_bd_pins hdmi_in/hdmi_vid_in]
connect_bd_net -net hdmi_vid_in_clk_0_1 [get_bd_ports hdmii_clk] [get_bd_pins hdmi_in/hdmi_vid_in_clk]

connect_bd_net -net proc_sys_reset_4_peripheral_aresetn [get_bd_pins axi_intc_0/s_axi_aresetn] [get_bd_pins hdmi_in/axis_resetn] [get_bd_pins hdmi_out/axis_resetn] [get_bd_pins hdmi_out_te0701/aresetn] [get_bd_pins proc_sys_reset_4/peripheral_aresetn] [get_bd_pins ps8_0_axi_periph/ARESETN] [get_bd_pins ps8_0_axi_periph/M06_ARESETN] [get_bd_pins ps8_0_axi_periph/M08_ARESETN] [get_bd_pins vcu_0/vcu_resetn]

connect_bd_net -net zynq_ultra_ps_e_0_pl_clk1 [get_bd_pins clk_wiz_0/clk_in1] [get_bd_pins clk_wiz_1/clk_in1] [get_bd_pins clk_wiz_2/clk_in1] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net -net zynq_ultra_ps_e_0_pl_resetn0 [get_bd_pins clk_wiz_0/resetn] [get_bd_pins clk_wiz_1/resetn] [get_bd_pins proc_sys_reset_1/ext_reset_in] [get_bd_pins proc_sys_reset_2/ext_reset_in] [get_bd_pins proc_sys_reset_3/ext_reset_in] [get_bd_pins proc_sys_reset_4/ext_reset_in] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]


startgroup
set_property -dict [list CONFIG.PSU__GPIO_EMIO__PERIPHERAL__ENABLE {1} CONFIG.PSU__GPIO_EMIO__PERIPHERAL__IO {1}] [get_bd_cells zynq_ultra_ps_e_0]
endgroup


connect_bd_net -net zynq_ultra_ps_e_0_emio_gpio_o [get_bd_ports im_resetn] [get_bd_pins zynq_ultra_ps_e_0/emio_gpio_o]

#add constraint
set fo [open ../constraints/_i_io.xdc a]
puts $fo "\n"

puts $fo "# ****************************************************************************"
puts $fo "# TE0720-02-2IF - TE0701-05 FMC - HDMIO (ADV7511) 12b DDR"
puts $fo "# ****************************************************************************"
puts $fo "set_property PACKAGE_PIN T6 \[get_ports {hdmioo_data\[0\]}\]"
puts $fo "set_property PACKAGE_PIN R6 \[get_ports {hdmioo_data\[1\]}\]"
puts $fo "set_property PACKAGE_PIN V9 \[get_ports {hdmioo_data\[2\]}\]"
puts $fo "set_property PACKAGE_PIN U9 \[get_ports {hdmioo_data\[3\]}\]"
puts $fo "set_property PACKAGE_PIN T7 \[get_ports {hdmioo_data\[4\]}\]"
puts $fo "set_property PACKAGE_PIN N8 \[get_ports {hdmioo_data\[5\]}\]"
puts $fo "set_property PACKAGE_PIN R7 \[get_ports {hdmioo_data\[6\]}\]"
puts $fo "set_property PACKAGE_PIN N9 \[get_ports {hdmioo_data\[7\]}\]"
puts $fo "set_property PACKAGE_PIN Y8 \[get_ports {hdmioo_data\[8\]}\]"
puts $fo "set_property PACKAGE_PIN V8 \[get_ports {hdmioo_data\[9\]}\]"
puts $fo "set_property PACKAGE_PIN W8 \[get_ports {hdmioo_data\[10\]}\]"
puts $fo "set_property PACKAGE_PIN U8 \[get_ports {hdmioo_data\[11\]}\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN L7 \[get_ports hdmioo_de\]"
puts $fo "set_property PACKAGE_PIN K4 \[get_ports hdmioo_hsync\]"
puts $fo "set_property PACKAGE_PIN K3 \[get_ports hdmioo_vsync\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN L6 \[get_ports hdmioo_clk\]"
puts $fo ""
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports hdmioo_*\]"
puts $fo "set_property SLEW FAST \[get_ports hdmioo_*\]"
puts $fo ""
puts $fo "create_clock -period 6.666 -name hdmioo_clk -waveform {0.000 3.333} -add \[get_ports hdmioo_clk\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN P7 \[get_ports hdmioo_i2c_scl_io\]"
puts $fo "set_property PACKAGE_PIN P6 \[get_ports hdmioo_i2c_sda_io\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports hdmioo_i2c_*\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN H7 \[get_ports {hdmioo_cec_clk\[0\]}\]"
puts $fo "set_property PACKAGE_PIN M8 \[get_ports {hdmioo_ct_hpd\[0\]}\]"
puts $fo "set_property PACKAGE_PIN J7 \[get_ports {hdmioo_ls_oe\[0\]}\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports {hdmioo_cec_clk\[0\]}\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports {hdmioo_ct_hpd\[0\]}\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports {hdmioo_ls_oe\[0\]}\]"
puts $fo ""
puts $fo ""
puts $fo "# ****************************************************************************"
puts $fo "# TE0820- TE0701- FMC - Imageon IIC"
puts $fo "# ****************************************************************************"
puts $fo "set_property PACKAGE_PIN F7 \[get_ports im_i2c_scl_io\]"
puts $fo "set_property PACKAGE_PIN G8 \[get_ports im_i2c_sda_io\]"
puts $fo ""
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports im_i2c_*\]"
puts $fo "set_property SLEW SLOW \[get_ports im_i2c_*\]"
puts $fo "set_property DRIVE 8 \[get_ports im_i2c_*\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN C4 \[get_ports {im_resetn\[0\]}\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports {im_resetn\[0\]}\]"
puts $fo "set_property SLEW SLOW \[get_ports {im_resetn\[0\]}\]"
puts $fo "set_property DRIVE 8 \[get_ports {im_resetn\[0\]}\]"
puts $fo ""
puts $fo "# ****************************************************************************"
puts $fo "# TE0820-- TE0701- FMC - Imageon HDMII (ADV7611)"
puts $fo "# ****************************************************************************"
puts $fo "set_property PACKAGE_PIN AD7  \[get_ports {hdmii\[15\]}\]"
puts $fo "set_property PACKAGE_PIN AE7  \[get_ports {hdmii\[14\]}\]"
puts $fo "set_property PACKAGE_PIN AH7  \[get_ports {hdmii\[13\]}\]"
puts $fo "set_property PACKAGE_PIN AH8  \[get_ports {hdmii\[12\]}\]"
puts $fo "set_property PACKAGE_PIN AG8  \[get_ports {hdmii\[11\]}\]"
puts $fo "set_property PACKAGE_PIN AF8  \[get_ports {hdmii\[10\]}\]"
puts $fo "set_property PACKAGE_PIN AH9  \[get_ports {hdmii\[9\]}\]"
puts $fo "set_property PACKAGE_PIN AG9  \[get_ports {hdmii\[8\]}\]"
puts $fo "set_property PACKAGE_PIN AG6  \[get_ports {hdmii\[7\]}\]"
puts $fo "set_property PACKAGE_PIN AE2  \[get_ports {hdmii\[6\]}\]"
puts $fo "set_property PACKAGE_PIN AC6  \[get_ports {hdmii\[5\]}\]"
puts $fo "set_property PACKAGE_PIN AG5  \[get_ports {hdmii\[4\]}\]"
puts $fo "set_property PACKAGE_PIN AF2  \[get_ports {hdmii\[3\]}\]"
puts $fo "set_property PACKAGE_PIN AB6  \[get_ports {hdmii\[2\]}\]"
puts $fo "set_property PACKAGE_PIN AC7  \[get_ports {hdmii\[1\]}\]"
puts $fo "set_property PACKAGE_PIN AB7  \[get_ports {hdmii\[0\]}\]"
puts $fo ""
puts $fo "# set_property PACKAGE_PIN U6   \[get_ports hdmii_spdif\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN D7  \[get_ports hdmii_clk\]"
puts $fo ""
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports hdmii\[*\]\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports hdmii_clk\]"
puts $fo ""
puts $fo "create_clock -period 6.666 -name hdmii_clk -waveform {0.000 3.333} -add \[get_ports hdmii_clk\]"
puts $fo ""
puts $fo "# ****************************************************************************"
puts $fo "# TE0820- - TE0701- FMC - Imageon HDMIO (ADV7511)"
puts $fo "# ****************************************************************************"
puts $fo "set_property PACKAGE_PIN AB2 \[get_ports {hdmio\[15\]}\]"
puts $fo "set_property PACKAGE_PIN AD5 \[get_ports {hdmio\[14\]}\]"
puts $fo "set_property PACKAGE_PIN AC2 \[get_ports {hdmio\[13\]}\]"
puts $fo "set_property PACKAGE_PIN AD2 \[get_ports {hdmio\[12\]}\]"
puts $fo "set_property PACKAGE_PIN AD4 \[get_ports {hdmio\[11\]}\]"
puts $fo "set_property PACKAGE_PIN AC4 \[get_ports {hdmio\[10\]}\]"
puts $fo "set_property PACKAGE_PIN AD1 \[get_ports {hdmio\[9\]}\]"
puts $fo "set_property PACKAGE_PIN AE3 \[get_ports {hdmio\[8\]}\]"
puts $fo "set_property PACKAGE_PIN AB1 \[get_ports {hdmio\[7\]}\]"
puts $fo "set_property PACKAGE_PIN AC3 \[get_ports {hdmio\[6\]}\]"
puts $fo "set_property PACKAGE_PIN AF3 \[get_ports {hdmio\[5\]}\]"
puts $fo "set_property PACKAGE_PIN AG4 \[get_ports {hdmio\[4\]}\]"
puts $fo "set_property PACKAGE_PIN AC1 \[get_ports {hdmio\[3\]}\]"
puts $fo "set_property PACKAGE_PIN AH4 \[get_ports {hdmio\[2\]}\]"
puts $fo "set_property PACKAGE_PIN AG3 \[get_ports {hdmio\[1\]}\]"
puts $fo "set_property PACKAGE_PIN AH3 \[get_ports {hdmio\[0\]}\]"
puts $fo ""
puts $fo "# set_property PACKAGE_PIN U5   \[get_ports hdmio_spdif\]"
puts $fo ""
puts $fo "set_property PACKAGE_PIN D6 \[get_ports hdmio_clk\]"
puts $fo ""
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports hdmio\[*\]\]"
puts $fo "set_property IOSTANDARD LVCMOS18 \[get_ports hdmio_clk\]"
puts $fo ""
puts $fo "create_clock -period 6.666 -name hdmio_clk -waveform {0.000 3.333} -add \[get_ports hdmio_clk\]"
puts $fo ""
puts $fo "# set asynchronous clock groups"
puts $fo "set_clock_groups -asynchronous -group \[get_clocks clk_out4_zusys_clk_wiz_0_0\] -group \[get_clocks hdmii_clk\]"

close $fo



# VDMAs end
#**********************************************


# add addresses to unmapped peripherals
# assign_bd_address

# Create address segments
assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces axi_vdma_0/Data_MM2S] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_DDR_LOW] -force
assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces axi_vdma_0/Data_MM2S] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_QSPI] -force
assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces axi_vdma_1/Data_MM2S] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_DDR_LOW] -force
assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces axi_vdma_1/Data_S2MM] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_DDR_LOW] -force
assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces axi_vdma_1/Data_MM2S] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_QSPI] -force
assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces axi_vdma_1/Data_S2MM] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_QSPI] -force
assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP2/HP0_DDR_LOW] -force
assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP2/HP0_DDR_LOW] -force
assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP2/HP0_LPS_OCM] -force
assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP2/HP0_LPS_OCM] -force
assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData0] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP2/HP0_QSPI] -force
assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/DecData1] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP2/HP0_QSPI] -force
assign_bd_address -offset 0x00000000 -range 0x80000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP0/HPC0_DDR_LOW] -force
assign_bd_address -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP0/HPC0_LPS_OCM] -force
assign_bd_address -offset 0xC0000000 -range 0x20000000 -target_address_space [get_bd_addr_spaces vcu_0/Code] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP0/HPC0_QSPI] -force
assign_bd_address -offset 0x80020000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_iic_0/S_AXI/Reg] -force
assign_bd_address -offset 0x80030000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_iic_1/S_AXI/Reg] -force
assign_bd_address -offset 0x80040000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_intc_0/S_AXI/Reg] -force
assign_bd_address -offset 0x80050000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_vdma_0/S_AXI_LITE/Reg] -force
assign_bd_address -offset 0x80060000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs axi_vdma_1/S_AXI_LITE/Reg] -force
assign_bd_address -offset 0x80080000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_in/axis_vid_det_0/S00_AXI/Reg] -force
assign_bd_address -offset 0x80070000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs clk_wiz_2/s_axi_lite/Reg] -force
assign_bd_address -offset 0x80090000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_in/hblank_det_0/S00_AXI/Reg] -force
assign_bd_address -offset 0x80000000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_in/v_proc_ss_0/s_axi_ctrl/Reg] -force
assign_bd_address -offset 0x80010000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_out/v_proc_ss_0/s_axi_ctrl/Reg] -force
assign_bd_address -offset 0x800A0000 -range 0x00010000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs hdmi_out/v_tc_0/ctrl/Reg] -force
assign_bd_address -offset 0x80100000 -range 0x00100000 -target_address_space [get_bd_addr_spaces zynq_ultra_ps_e_0/Data] [get_bd_addr_segs vcu_0/S_AXI_LITE/Reg] -force

# Exclude Address Segments
exclude_bd_addr_seg -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces axi_vdma_0/Data_MM2S] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_LPS_OCM]
exclude_bd_addr_seg -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces axi_vdma_1/Data_MM2S] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_LPS_OCM]
exclude_bd_addr_seg -offset 0xFF000000 -range 0x01000000 -target_address_space [get_bd_addr_spaces axi_vdma_1/Data_S2MM] [get_bd_addr_segs zynq_ultra_ps_e_0/SAXIGP3/HP1_LPS_OCM]


#save
save_bd_design

#save project XPR name
global proj_xpr
set proj_xpr [current_project]
append proj_xpr .xpr

#close project
close_project

# reopen project
open_project $proj_xpr

# open block design
open_bd_design [current_project].srcs/sources_1/bd/zusys/zusys.bd

#validate
#validate_bd_design
